diff -pruN 0.7.13-3/Announce.txt 0.7.18-1/Announce.txt
--- 0.7.13-3/Announce.txt	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/Announce.txt	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,135 @@
+Subject: [ANN] god 0.6.0 released (and mailing list)
+
+!!!NEW MAILING LIST!!! We now have a mailing list at http://groups.google.com/group/god-rb
+
+This release is primarily focused on increased stability, robustness, and code cleanliness.
+
+The last release (0.5.0) switched from TCP sockets to Unix Domain Sockets for the CLI tools. There were some issues regarding file descriptors that would cause god to hang when restarting (the unix socket was being held open by spawned processes). These problems are all fixed in 0.6.0. You may need to kill any processes that were started under god 0.5.0 when you upgrade to 0.6.0 (you will only need to do this once during the upgrade process).
+
+More attention is now paid to Syslogging behavior.
+
+God will delete its own PID file when terminated via a user request.
+
+A new command line option now allows you to check whether your installation properly handles events (sometimes the C extension will compile ok, but still not support events. Gentoo, I'm looking at you). Run `god check` to see an attempt to use the event system. It will tell you if your installation does not support events.
+
+A watch can now be removed entirely at runtime using `god remove <watch name>`.
+
+A new condition DiskUsage allows you to check the available disk space on a given volume.
+
+Updated documentation is now available on the website:
+
+  http://god.rubyforge.org/
+
+
+WHAT IS GOD?
+
+God is an easy to configure, easy to extend monitoring framework written in Ruby.
+
+Keeping your server processes and tasks running should be a simple part of your deployment process. God aims to be the simplest, most powerful monitoring application available.
+
+
+DISCLAIMER
+
+God is still beta so I do not yet recommend you use it for mission critical tasks. We are using it at Powerset, Inc. to monitor our public facing applications, but then, we're daring fellows.
+
+
+INSTALL
+
+sudo gem install god
+
+
+FEATURES
+
+* Config file is written in Ruby
+* Easily write your own custom conditions in Ruby
+* Supports both poll and event based conditions
+* Different poll conditions can have different intervals
+* Easily control non-daemonized processes
+
+
+EXAMPLE
+
+The easiest way to understand how god will make your life better is by looking at a sample config file. The following configuration file is what I use at gravatar.com to keep the mongrels running:
+
+# run with:  god -c /path/to/gravatar.god
+# 
+# This is the actual config file used to keep the mongrels of
+# gravatar.com running.
+
+RAILS_ROOT = "/Users/tom/dev/gravatar2"
+
+%w{8200 8201 8202}.each do |port|
+  God.watch do |w|
+    w.name = "gravatar2-mongrel-#{port}"
+    w.interval = 30.seconds # default      
+    w.start = "mongrel_rails start -c #{RAILS_ROOT} -p #{port} \
+      -P #{RAILS_ROOT}/log/mongrel.#{port}.pid  -d"
+    w.stop = "mongrel_rails stop -P #{RAILS_ROOT}/log/mongrel.#{port}.pid"
+    w.restart = "mongrel_rails restart -P #{RAILS_ROOT}/log/mongrel.#{port}.pid"
+    w.start_grace = 10.seconds
+    w.restart_grace = 10.seconds
+    w.pid_file = File.join(RAILS_ROOT, "log/mongrel.#{port}.pid")
+    
+    w.behavior(:clean_pid_file)
+
+    w.start_if do |start|
+      start.condition(:process_running) do |c|
+        c.interval = 5.seconds
+        c.running = false
+      end
+    end
+    
+    w.restart_if do |restart|
+      restart.condition(:memory_usage) do |c|
+        c.above = 150.megabytes
+        c.times = [3, 5] # 3 out of 5 intervals
+      end
+    
+      restart.condition(:cpu_usage) do |c|
+        c.above = 50.percent
+        c.times = 5
+      end
+    end
+    
+    # lifecycle
+    w.lifecycle do |on|
+      on.condition(:flapping) do |c|
+        c.to_state = [:start, :restart]
+        c.times = 5
+        c.within = 5.minute
+        c.transition = :unmonitored
+        c.retry_in = 10.minutes
+        c.retry_times = 5
+        c.retry_within = 2.hours
+      end
+    end
+  end
+end
+
+
+DOCS
+
+Detailed documentation is available at http://god.rubyforge.org/
+
+
+CHANGES
+
+== 0.6.0 / 2007-12-4
+
+* Minor Enhancement
+  * Move Syslog calls into God::Logger and clean up all calling code
+  * Remove god's pid file on user requested termination
+  * Better handling and cleanup of DRb server's unix domain socket
+  * Allow shorthand for requesting a god log
+  * Add `god check` to make it easier to diagnose event problems
+  * Refactor god binary into class/method structure
+  * Implement `god remove` to remove a Task altogether
+* New Conditions
+  * DiskUsage < PollCondition - trigger if disk usage is above limit on mount [Rudy Desjardins]
+
+
+
+AUTHORS
+
+Tom Preston-Werner
+Kevin Clark
diff -pruN 0.7.13-3/bin/god 0.7.18-1/bin/god
--- 0.7.13-3/bin/god	2010-02-22 00:39:50.000000000 +0000
+++ 0.7.18-1/bin/god	2010-02-21 14:01:37.000000000 +0000
@@ -7,6 +7,7 @@ $:.unshift File.join(File.dirname(__FILE
 require 'rubygems'
 require 'optparse'
 require 'drb'
+require 'yaml'
 
 begin
   options = {:daemonize => true, :port => 17165, :syslog => true, :events => true}
Binary files 0.7.13-3/bin/._god and 0.7.18-1/bin/._god differ
diff -pruN 0.7.13-3/.config-god 0.7.18-1/.config-god
--- 0.7.13-3/.config-god	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/.config-god	2010-02-21 14:03:15.000000000 +0000
@@ -0,0 +1,20 @@
+prefix=/usr
+bindir=$prefix/bin
+libdir=$prefix/lib
+datadir=$prefix/share
+mandir=$prefix/share/man
+sysconfdir=/etc
+localstatedir=/var
+libruby=/usr/lib/ruby
+librubyver=/usr/lib/ruby/1.8
+librubyverarch=/usr/lib/ruby/1.8/i486-linux
+siteruby=/usr/local/lib/site_ruby
+siterubyver=/usr/local/lib/site_ruby/1.8
+siterubyverarch=/usr/local/lib/site_ruby/1.8/i486-linux
+rbdir=$librubyver
+sodir=$librubyverarch
+rubypath=/usr/bin/ruby1.8
+rubyprog=/usr/bin/ruby1.8
+makeprog=make
+shebang=ruby
+without-ext=no
diff -pruN 0.7.13-3/debian/changelog 0.7.18-1/debian/changelog
--- 0.7.13-3/debian/changelog	2010-02-22 00:39:50.000000000 +0000
+++ 0.7.18-1/debian/changelog	2010-02-22 00:39:50.000000000 +0000
@@ -1,3 +1,11 @@
+god (0.7.18-1) unstable; urgency=low
+
+  * New upstream release.
+  * Updated watch file.
+  * Bumped up Standards-Version.
+
+ -- Sebastien Delafond <seb@debian.org>  Sun, 21 Feb 2010 14:52:42 +0100
+
 god (0.7.13-3) unstable; urgency=low
 
   * Really depend on rubygems (Closes: #562447).
diff -pruN 0.7.13-3/debian/control 0.7.18-1/debian/control
--- 0.7.13-3/debian/control	2010-02-22 00:39:50.000000000 +0000
+++ 0.7.18-1/debian/control	2010-02-22 00:39:50.000000000 +0000
@@ -3,7 +3,7 @@ Section: interpreters
 Priority: optional
 Maintainer: Sebastien Delafond <seb@debian.org>
 Build-Depends: debhelper (>= 5), cdbs, ruby1.8-dev, ruby1.8, ruby-pkg-tools
-Standards-Version: 3.8.3
+Standards-Version: 3.8.4
 Homepage: http://god.rubyforge.org/
 
 Package: god
diff -pruN 0.7.13-3/debian/watch 0.7.18-1/debian/watch
--- 0.7.13-3/debian/watch	2010-02-22 00:39:50.000000000 +0000
+++ 0.7.18-1/debian/watch	2010-02-22 00:39:50.000000000 +0000
@@ -1,2 +1,2 @@
 version=3
-http://rubyforge.org/frs/?group_id=3845 .*god-([0-9,.].*)\.tgz
+http://rubyforge.org/frs/?group_id=3845 .*god-([0-9,.].*)\.gem
Binary files 0.7.13-3/ext/god/kqueue_handler.o and 0.7.18-1/ext/god/kqueue_handler.o differ
diff -pruN 0.7.13-3/ext/god/Makefile 0.7.18-1/ext/god/Makefile
--- 0.7.13-3/ext/god/Makefile	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/ext/god/Makefile	2010-02-21 14:02:04.000000000 +0000
@@ -0,0 +1,157 @@
+
+SHELL = /bin/sh
+
+#### Start of system configuration section. ####
+
+srcdir = /home/seb/work/debian/god/god-0.7.18/ext/god
+topdir = /usr/lib/ruby/1.8/i486-linux
+hdrdir = $(topdir)
+VPATH = $(srcdir):$(topdir):$(hdrdir)
+exec_prefix = $(prefix)
+prefix = $(DESTDIR)/usr
+sharedstatedir = $(prefix)/com
+mandir = $(prefix)/share/man
+psdir = $(docdir)
+oldincludedir = $(DESTDIR)/usr/include
+localedir = $(datarootdir)/locale
+bindir = $(exec_prefix)/bin
+libexecdir = $(prefix)/lib/ruby1.8
+sitedir = $(DESTDIR)/usr/local/lib/site_ruby
+htmldir = $(docdir)
+vendorarchdir = $(vendorlibdir)/$(sitearch)
+includedir = $(prefix)/include
+infodir = $(prefix)/share/info
+vendorlibdir = $(vendordir)/$(ruby_version)
+sysconfdir = $(DESTDIR)/etc
+libdir = $(exec_prefix)/lib
+sbindir = $(exec_prefix)/sbin
+rubylibdir = $(libdir)/ruby/$(ruby_version)
+docdir = $(datarootdir)/doc/$(PACKAGE)
+dvidir = $(docdir)
+vendordir = $(libdir)/ruby/vendor_ruby
+datarootdir = $(prefix)/share
+pdfdir = $(docdir)
+archdir = $(rubylibdir)/$(arch)
+sitearchdir = $(sitelibdir)/$(sitearch)
+datadir = $(datarootdir)
+localstatedir = $(DESTDIR)/var
+sitelibdir = $(sitedir)/$(ruby_version)
+
+CC = cc
+LIBRUBY = $(LIBRUBY_SO)
+LIBRUBY_A = lib$(RUBY_SO_NAME)-static.a
+LIBRUBYARG_SHARED = -l$(RUBY_SO_NAME)
+LIBRUBYARG_STATIC = -l$(RUBY_SO_NAME)-static
+
+RUBY_EXTCONF_H = 
+CFLAGS   =  -fPIC -fno-strict-aliasing -g -g -O2  -fPIC $(cflags) 
+INCFLAGS = -I. -I$(topdir) -I$(hdrdir) -I$(srcdir)
+DEFS     = -D_FILE_OFFSET_BITS=64
+CPPFLAGS = -DHAVE_LINUX_NETLINK_H -DHAVE_LINUX_CONNECTOR_H -DHAVE_LINUX_CN_PROC_H  $(DEFS) $(cppflags)
+CXXFLAGS = $(CFLAGS) 
+ldflags  = -L.  -rdynamic -Wl,-export-dynamic
+dldflags = 
+archflag = 
+DLDFLAGS = $(ldflags) $(dldflags) $(archflag)
+LDSHARED = $(CC) -shared
+AR = ar
+EXEEXT = 
+
+RUBY_INSTALL_NAME = ruby1.8
+RUBY_SO_NAME = ruby1.8
+arch = i486-linux
+sitearch = i486-linux
+ruby_version = 1.8
+ruby = /usr/bin/ruby1.8
+RUBY = $(ruby)
+RM = rm -f
+MAKEDIRS = mkdir -p
+INSTALL = /usr/bin/install -c
+INSTALL_PROG = $(INSTALL) -m 0755
+INSTALL_DATA = $(INSTALL) -m 644
+COPY = cp
+
+#### End of system configuration section. ####
+
+preload = 
+
+libpath = . $(libdir)
+LIBPATH =  -L. -L$(libdir)
+DEFFILE = 
+
+CLEANFILES = mkmf.log
+DISTCLEANFILES = 
+
+extout = 
+extout_prefix = 
+target_prefix = 
+LOCAL_LIBS = 
+LIBS = $(LIBRUBYARG_SHARED)  -lpthread -lrt -ldl -lcrypt -lm   -lc
+SRCS = kqueue_handler.c netlink_handler.c
+OBJS = kqueue_handler.o netlink_handler.o
+TARGET = netlink_handler_ext
+DLLIB = $(TARGET).so
+EXTSTATIC = 
+STATIC_LIB = 
+
+BINDIR        = $(bindir)
+RUBYCOMMONDIR = $(sitedir)$(target_prefix)
+RUBYLIBDIR    = $(sitelibdir)$(target_prefix)
+RUBYARCHDIR   = $(sitearchdir)$(target_prefix)
+
+TARGET_SO     = $(DLLIB)
+CLEANLIBS     = $(TARGET).so $(TARGET).il? $(TARGET).tds $(TARGET).map
+CLEANOBJS     = *.o *.a *.s[ol] *.pdb *.exp *.bak
+
+all:		$(DLLIB)
+static:		$(STATIC_LIB)
+
+clean:
+		@-$(RM) $(CLEANLIBS) $(CLEANOBJS) $(CLEANFILES)
+
+distclean:	clean
+		@-$(RM) Makefile $(RUBY_EXTCONF_H) conftest.* mkmf.log
+		@-$(RM) core ruby$(EXEEXT) *~ $(DISTCLEANFILES)
+
+realclean:	distclean
+install: install-so install-rb
+
+install-so: $(RUBYARCHDIR)
+install-so: $(RUBYARCHDIR)/$(DLLIB)
+$(RUBYARCHDIR)/$(DLLIB): $(DLLIB)
+	$(INSTALL_PROG) $(DLLIB) $(RUBYARCHDIR)
+install-rb: pre-install-rb install-rb-default
+install-rb-default: pre-install-rb-default
+pre-install-rb: Makefile
+pre-install-rb-default: Makefile
+$(RUBYARCHDIR):
+	$(MAKEDIRS) $@
+
+site-install: site-install-so site-install-rb
+site-install-so: install-so
+site-install-rb: install-rb
+
+.SUFFIXES: .c .m .cc .cxx .cpp .C .o
+
+.cc.o:
+	$(CXX) $(INCFLAGS) $(CPPFLAGS) $(CXXFLAGS) -c $<
+
+.cxx.o:
+	$(CXX) $(INCFLAGS) $(CPPFLAGS) $(CXXFLAGS) -c $<
+
+.cpp.o:
+	$(CXX) $(INCFLAGS) $(CPPFLAGS) $(CXXFLAGS) -c $<
+
+.C.o:
+	$(CXX) $(INCFLAGS) $(CPPFLAGS) $(CXXFLAGS) -c $<
+
+.c.o:
+	$(CC) $(INCFLAGS) $(CPPFLAGS) $(CFLAGS) -c $<
+
+$(DLLIB): $(OBJS) Makefile
+	@-$(RM) $@
+	$(LDSHARED) -o $@ $(OBJS) $(LIBPATH) $(DLDFLAGS) $(LOCAL_LIBS) $(LIBS)
+
+
+
+$(OBJS): ruby.h defines.h
diff -pruN 0.7.13-3/ext/god/mkmf.log 0.7.18-1/ext/god/mkmf.log
--- 0.7.13-3/ext/god/mkmf.log	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/ext/god/mkmf.log	2010-02-21 14:02:04.000000000 +0000
@@ -0,0 +1,30 @@
+have_header: checking for linux/netlink.h... -------------------- yes
+
+"cc -E -I. -I/usr/lib/ruby/1.8/i486-linux -I/home/seb/work/debian/god/god-0.7.18/ext/god  -D_FILE_OFFSET_BITS=64  -fno-strict-aliasing -g -g -O2  -fPIC    conftest.c -o conftest.i"
+checked program was:
+/* begin */
+1: #include <linux/netlink.h>
+/* end */
+
+--------------------
+
+have_header: checking for linux/connector.h... -------------------- yes
+
+"cc -E -I. -I/usr/lib/ruby/1.8/i486-linux -I/home/seb/work/debian/god/god-0.7.18/ext/god  -D_FILE_OFFSET_BITS=64  -fno-strict-aliasing -g -g -O2  -fPIC    conftest.c -o conftest.i"
+checked program was:
+/* begin */
+1: #include <linux/connector.h>
+/* end */
+
+--------------------
+
+have_header: checking for linux/cn_proc.h... -------------------- yes
+
+"cc -E -I. -I/usr/lib/ruby/1.8/i486-linux -I/home/seb/work/debian/god/god-0.7.18/ext/god  -D_FILE_OFFSET_BITS=64  -fno-strict-aliasing -g -g -O2  -fPIC    conftest.c -o conftest.i"
+checked program was:
+/* begin */
+1: #include <linux/cn_proc.h>
+/* end */
+
+--------------------
+
diff -pruN 0.7.13-3/ext/god/netlink_handler.c 0.7.18-1/ext/god/netlink_handler.c
--- 0.7.13-3/ext/god/netlink_handler.c	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/ext/god/netlink_handler.c	2010-02-21 14:01:37.000000000 +0000
@@ -6,6 +6,7 @@
 #include <sys/socket.h>
 #include <linux/netlink.h>
 #include <linux/connector.h>
+#define _LINUX_TIME_H
 #include <linux/cn_proc.h>
 #include <errno.h>
 
Binary files 0.7.13-3/ext/god/netlink_handler_ext.so and 0.7.18-1/ext/god/netlink_handler_ext.so differ
Binary files 0.7.13-3/ext/god/netlink_handler.o and 0.7.18-1/ext/god/netlink_handler.o differ
Binary files 0.7.13-3/god_0.7.18.orig.tar.gz and 0.7.18-1/god_0.7.18.orig.tar.gz differ
diff -pruN 0.7.13-3/god.gemspec 0.7.18-1/god.gemspec
--- 0.7.13-3/god.gemspec	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/god.gemspec	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,223 @@
+# -*- encoding: utf-8 -*-
+
+Gem::Specification.new do |s|
+  s.name = %q{god}
+  s.version = "0.7.18"
+
+  s.required_rubygems_version = Gem::Requirement.new(">= 0") if s.respond_to? :required_rubygems_version=
+  s.authors = ["Tom Preston-Werner"]
+  s.date = %q{2009-09-10}
+  s.default_executable = %q{god}
+  s.description = %q{God is an easy to configure, easy to extend monitoring framework written in Ruby.}
+  s.email = %q{tom@mojombo.com}
+  s.executables = ["god"]
+  s.extensions = ["ext/god/extconf.rb"]
+  s.extra_rdoc_files = [
+    "README.txt"
+  ]
+  s.files = [
+    ".gitignore",
+     "Announce.txt",
+     "History.txt",
+     "README.txt",
+     "Rakefile",
+     "VERSION.yml",
+     "bin/god",
+     "examples/events.god",
+     "examples/gravatar.god",
+     "examples/single.god",
+     "ext/god/.gitignore",
+     "ext/god/extconf.rb",
+     "ext/god/kqueue_handler.c",
+     "ext/god/netlink_handler.c",
+     "god.gemspec",
+     "ideas/execve/.DS_Store",
+     "ideas/execve/execve.c",
+     "ideas/execve/extconf.rb",
+     "ideas/execve/go.rb",
+     "ideas/future.god",
+     "init/god",
+     "init/lsb_compliant_god",
+     "lib/god.rb",
+     "lib/god/behavior.rb",
+     "lib/god/behaviors/clean_pid_file.rb",
+     "lib/god/behaviors/clean_unix_socket.rb",
+     "lib/god/behaviors/notify_when_flapping.rb",
+     "lib/god/cli/command.rb",
+     "lib/god/cli/run.rb",
+     "lib/god/cli/version.rb",
+     "lib/god/condition.rb",
+     "lib/god/conditions/always.rb",
+     "lib/god/conditions/complex.rb",
+     "lib/god/conditions/cpu_usage.rb",
+     "lib/god/conditions/degrading_lambda.rb",
+     "lib/god/conditions/disk_usage.rb",
+     "lib/god/conditions/file_mtime.rb",
+     "lib/god/conditions/flapping.rb",
+     "lib/god/conditions/http_response_code.rb",
+     "lib/god/conditions/lambda.rb",
+     "lib/god/conditions/memory_usage.rb",
+     "lib/god/conditions/process_exits.rb",
+     "lib/god/conditions/process_running.rb",
+     "lib/god/conditions/tries.rb",
+     "lib/god/configurable.rb",
+     "lib/god/contact.rb",
+     "lib/god/contacts/campfire.rb",
+     "lib/god/contacts/email.rb",
+     "lib/god/contacts/jabber.rb",
+     "lib/god/contacts/twitter.rb",
+     "lib/god/contacts/webhook.rb",
+     "lib/god/dependency_graph.rb",
+     "lib/god/diagnostics.rb",
+     "lib/god/driver.rb",
+     "lib/god/errors.rb",
+     "lib/god/event_handler.rb",
+     "lib/god/event_handlers/dummy_handler.rb",
+     "lib/god/event_handlers/kqueue_handler.rb",
+     "lib/god/event_handlers/netlink_handler.rb",
+     "lib/god/logger.rb",
+     "lib/god/metric.rb",
+     "lib/god/process.rb",
+     "lib/god/registry.rb",
+     "lib/god/simple_logger.rb",
+     "lib/god/socket.rb",
+     "lib/god/sugar.rb",
+     "lib/god/system/portable_poller.rb",
+     "lib/god/system/process.rb",
+     "lib/god/system/slash_proc_poller.rb",
+     "lib/god/task.rb",
+     "lib/god/timeline.rb",
+     "lib/god/trigger.rb",
+     "lib/god/watch.rb",
+     "site/images/banner.jpg",
+     "site/images/bg.gif",
+     "site/images/bg_grey.gif",
+     "site/images/bullet.jpg",
+     "site/images/corner_green.gif",
+     "site/images/corner_green.psd",
+     "site/images/corner_pink.gif",
+     "site/images/god_logo.png",
+     "site/images/header_bg.gif",
+     "site/images/header_bg.jpg",
+     "site/images/red_dot.gif",
+     "site/images/top_bg.gif",
+     "site/index.html",
+     "site/install.html",
+     "site/javascripts/code_highlighter.js",
+     "site/javascripts/ruby.js",
+     "site/stylesheets/layout.css",
+     "test/configs/child_events/child_events.god",
+     "test/configs/child_events/simple_server.rb",
+     "test/configs/child_polls/child_polls.god",
+     "test/configs/child_polls/simple_server.rb",
+     "test/configs/complex/complex.god",
+     "test/configs/complex/simple_server.rb",
+     "test/configs/contact/contact.god",
+     "test/configs/contact/simple_server.rb",
+     "test/configs/daemon_events/daemon_events.god",
+     "test/configs/daemon_events/simple_server.rb",
+     "test/configs/daemon_events/simple_server_stop.rb",
+     "test/configs/daemon_polls/daemon_polls.god",
+     "test/configs/daemon_polls/simple_server.rb",
+     "test/configs/degrading_lambda/degrading_lambda.god",
+     "test/configs/degrading_lambda/tcp_server.rb",
+     "test/configs/lifecycle/lifecycle.god",
+     "test/configs/matias/matias.god",
+     "test/configs/real.rb",
+     "test/configs/running_load/running_load.god",
+     "test/configs/stress/simple_server.rb",
+     "test/configs/stress/stress.god",
+     "test/configs/task/logs/.placeholder",
+     "test/configs/task/task.god",
+     "test/configs/test.rb",
+     "test/helper.rb",
+     "test/suite.rb",
+     "test/test_behavior.rb",
+     "test/test_campfire.rb",
+     "test/test_condition.rb",
+     "test/test_conditions_disk_usage.rb",
+     "test/test_conditions_http_response_code.rb",
+     "test/test_conditions_process_running.rb",
+     "test/test_conditions_tries.rb",
+     "test/test_contact.rb",
+     "test/test_dependency_graph.rb",
+     "test/test_driver.rb",
+     "test/test_email.rb",
+     "test/test_event_handler.rb",
+     "test/test_god.rb",
+     "test/test_handlers_kqueue_handler.rb",
+     "test/test_jabber.rb",
+     "test/test_logger.rb",
+     "test/test_metric.rb",
+     "test/test_process.rb",
+     "test/test_registry.rb",
+     "test/test_socket.rb",
+     "test/test_sugar.rb",
+     "test/test_system_portable_poller.rb",
+     "test/test_system_process.rb",
+     "test/test_task.rb",
+     "test/test_timeline.rb",
+     "test/test_trigger.rb",
+     "test/test_watch.rb",
+     "test/test_webhook.rb"
+  ]
+  s.homepage = %q{http://god.rubyforge.org/}
+  s.rdoc_options = ["--charset=UTF-8"]
+  s.require_paths = ["lib", "ext"]
+  s.rubyforge_project = %q{god}
+  s.rubygems_version = %q{1.3.5}
+  s.summary = %q{Like monit, only awesome}
+  s.test_files = [
+    "test/configs/child_events/simple_server.rb",
+     "test/configs/child_polls/simple_server.rb",
+     "test/configs/complex/simple_server.rb",
+     "test/configs/contact/simple_server.rb",
+     "test/configs/daemon_events/simple_server.rb",
+     "test/configs/daemon_events/simple_server_stop.rb",
+     "test/configs/daemon_polls/simple_server.rb",
+     "test/configs/degrading_lambda/tcp_server.rb",
+     "test/configs/real.rb",
+     "test/configs/stress/simple_server.rb",
+     "test/configs/test.rb",
+     "test/helper.rb",
+     "test/suite.rb",
+     "test/test_behavior.rb",
+     "test/test_campfire.rb",
+     "test/test_condition.rb",
+     "test/test_conditions_disk_usage.rb",
+     "test/test_conditions_http_response_code.rb",
+     "test/test_conditions_process_running.rb",
+     "test/test_conditions_tries.rb",
+     "test/test_contact.rb",
+     "test/test_dependency_graph.rb",
+     "test/test_driver.rb",
+     "test/test_email.rb",
+     "test/test_event_handler.rb",
+     "test/test_god.rb",
+     "test/test_handlers_kqueue_handler.rb",
+     "test/test_jabber.rb",
+     "test/test_logger.rb",
+     "test/test_metric.rb",
+     "test/test_process.rb",
+     "test/test_registry.rb",
+     "test/test_socket.rb",
+     "test/test_sugar.rb",
+     "test/test_system_portable_poller.rb",
+     "test/test_system_process.rb",
+     "test/test_task.rb",
+     "test/test_timeline.rb",
+     "test/test_trigger.rb",
+     "test/test_watch.rb",
+     "test/test_webhook.rb"
+  ]
+
+  if s.respond_to? :specification_version then
+    current_version = Gem::Specification::CURRENT_SPECIFICATION_VERSION
+    s.specification_version = 3
+
+    if Gem::Version.new(Gem::RubyGemsVersion) >= Gem::Version.new('1.2.0') then
+    else
+    end
+  else
+  end
+end
Binary files 0.7.13-3/._History.txt and 0.7.18-1/._History.txt differ
diff -pruN 0.7.13-3/History.txt 0.7.18-1/History.txt
--- 0.7.13-3/History.txt	2009-05-04 08:54:29.000000000 +0100
+++ 0.7.18-1/History.txt	2010-02-21 14:01:37.000000000 +0000
@@ -1,3 +1,39 @@
+== 0.7.18 / 2009-09-09
+  * Minor Enhancements
+    * Better handling of unexpected exceptions in conditions
+    * Added support for running processes in a directory other than '/' [github.com/samhendley]
+  * Bug Fixes
+    * Generate an actual unique identifier for email contact [github.com/underley]
+
+== 0.7.17 / 2009-08-25
+  * Bug Fixes
+    * Fix the glob and directory config loading for -c option
+
+== 0.7.16 / 2009-08-24
+  * Minor Enhancements
+    * Better logging for disk_usage condition [github.com/lettherebecode]
+  * Bug Fixes
+    * Only sleep if driver delay is > 0 [github.com/ps2]
+    * Rescue Timeout::Error exception due to smtp server timing out [github.com/ps2]
+    * Disk usage condition should use `df -P` to prevent line splitting [github.com/mseppae]
+    * Always require YAML so binary works on dumb systems
+
+== 0.7.15 / 2009-08-19
+  * Minor Enhancements
+    * Support SSL Campfire connections [github.com/eric]
+    * Allow wildcards in -c configuration file option
+
+== 0.7.14 / 2009-08-10
+  * Minor Enhancements
+    * Only store log lines when a client wishes to see them
+    * Add an lsb-compliant init script into god/init [Woody Peterson]
+    * Never require stop command; use default killer if none is specified
+  * Bug Fixes
+    * Fix redefinition error for time.h and allow it to compile on Ubuntu Edgy [github.com/tbuser]
+    * Fix a memory leak in jabber by adding a call to jabber_client.close [github.com/woahdae]
+    * Make jabber code manage one connection to make it faster, use less memory,
+      and not leak [github.com/woahdae]
+
 == 0.7.13 / 2009-05-04
   * Bug Fixes
     * Auto daemonized processes are now stopped/unmonitored correctly [github.com/jcapote]
Binary files 0.7.13-3/ideas/execve/.DS_Store and 0.7.18-1/ideas/execve/.DS_Store differ
diff -pruN 0.7.13-3/ideas/execve/execve.c 0.7.18-1/ideas/execve/execve.c
--- 0.7.13-3/ideas/execve/execve.c	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/ideas/execve/execve.c	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,29 @@
+#include "ruby.h"
+
+static VALUE mKernel;
+void Init_execve();
+VALUE method_execve(VALUE self, VALUE cmd, VALUE env);
+
+void Init_execve() {
+  mKernel = rb_const_get(rb_cObject, rb_intern("Kernel"));
+  rb_define_method(mKernel, "execve", method_execve, 2);
+}
+
+VALUE method_execve(VALUE self, VALUE r_cmd, VALUE r_env) {
+  char *shell = (char *)dln_find_exe("sh", 0);
+  char *arg[] = { "sh", "-c", StringValuePtr(r_cmd), (char *)0 };
+  
+  struct RArray *env_array;
+  env_array = RARRAY(r_env);
+  char *env[env_array->len + 1];
+  
+  int i;
+  for(i = 0; i < env_array->len; i++) {
+    env[i] = StringValuePtr(env_array->ptr[i]);
+  }
+  
+  env[env_array->len] = (char *)0;
+  
+  execve(shell, arg, env);
+  return Qnil;
+}
\ No newline at end of file
diff -pruN 0.7.13-3/ideas/execve/extconf.rb 0.7.18-1/ideas/execve/extconf.rb
--- 0.7.13-3/ideas/execve/extconf.rb	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/ideas/execve/extconf.rb	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,11 @@
+# Loads mkmf which is used to make makefiles for Ruby extensions
+require 'mkmf'
+
+# Give it a name
+extension_name = 'execve'
+
+# The destination
+dir_config(extension_name)
+
+# Do the work
+create_makefile(extension_name)
\ No newline at end of file
diff -pruN 0.7.13-3/ideas/execve/go.rb 0.7.18-1/ideas/execve/go.rb
--- 0.7.13-3/ideas/execve/go.rb	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/ideas/execve/go.rb	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,8 @@
+require 'execve'
+
+my_env = ENV.to_hash.merge('HOME' => '/foo')
+# my_env = ENV.to_hash
+
+env = my_env.keys.inject([]) { |acc, k| acc << "#{k}=#{my_env[k]}"; acc }
+
+execve(%Q{ruby -e "puts ENV['HOME']"}, env)
diff -pruN 0.7.13-3/ideas/future.god 0.7.18-1/ideas/future.god
--- 0.7.13-3/ideas/future.god	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/ideas/future.god	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,82 @@
+# This example shows how you might keep a local development Rails server up
+# and running on your Mac.
+
+# Run with:
+# god -c /path/to/events.god
+
+RAILS_ROOT = "/Users/tom/dev/helloworld"
+
+God::Contacts::Email.delivery_method = :smtp
+God::Contacts::Email.server_settings = {}
+
+God.contact(:email) do |c|
+  c.name = 'tom'
+  c.email = 'tom@powerset.com'
+  c.group = 'developers'
+  c.throttle = 30.minutes
+end
+
+God.watch do |w|
+  w.name = "local-3000"
+  w.interval = 5.seconds
+  w.start = "mongrel_rails start -P ./log/mongrel.pid -c #{RAILS_ROOT} -d"
+  w.stop = "mongrel_rails stop -P ./log/mongrel.pid -c #{RAILS_ROOT}"
+  w.pid_file = File.join(RAILS_ROOT, "log/mongrel.pid")
+  
+  # clean pid files before start if necessary
+  w.behavior(:clean_pid_file)
+  
+  # determine the state on startup
+  w.transition(:init, { true => :up, false => :start }) do |on|
+    on.condition(:process_running) do |c|
+      c.running = true
+    end
+  end
+  
+  # determine when process has finished starting
+  w.transition([:start, :restart], :up) do |on|
+    on.condition(:process_running) do |c|
+      c.running = true
+    end
+  end
+
+  # start if process is not running
+  w.transition(:up, :start) do |on|
+    on.condition(:process_exits)
+  end
+  
+  # restart if memory or cpu is too high
+  w.transition(:up, :restart) do |on|
+    on.condition(:memory_usage) do |c|
+      c.interval = 20
+      c.above = 50.megabytes
+      c.times = [3, 5]
+    end
+    
+    on.condition(:cpu_usage) do |c|
+      c.interval = 10
+      c.above = 10.percent
+      c.times = [3, 5]
+    end
+  end
+  
+  # lifecycle
+  w.lifecycle do |on|
+    on.condition(:flapping) do |c|
+      c.to_state = [:start, :restart]
+      c.times = 5
+      c.within = 30.seconds
+      c.transition = nil
+      c.retry = 60.seconds
+      c.giveup_tries = 5
+      c.notify = 'tom'
+    end
+    
+    on.condition(:memory_usage) do |c|
+      c.interval = 20
+      c.above = 40.megabytes
+      c.times = [3, 5]
+      c.notify = 'tom'
+    end
+  end
+end
\ No newline at end of file
diff -pruN 0.7.13-3/init/lsb_compliant_god 0.7.18-1/init/lsb_compliant_god
--- 0.7.13-3/init/lsb_compliant_god	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/init/lsb_compliant_god	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,109 @@
+#!/usr/bin/env ruby
+
+# see http://refspecs.freestandards.org/LSB_3.1.0/LSB-Core-generic/LSB-Core-generic/iniscrptact.html
+
+PATH="/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin"
+DESC="god daemon"
+NAME="god"
+DAEMON="/usr/bin/#{NAME}"
+CONFIGFILE="/etc/god/god.conf"
+PIDFILE="/var/run/#{NAME}.pid"
+SCRIPTNAME="/etc/init.d/#{NAME}"
+START_FLAGS="--no-syslog -P #{PIDFILE}"
+
+ENV["PATH"] = PATH
+
+def read_pid
+  begin
+    @pid = File.read(PIDFILE).to_i
+    @pid = nil if @pid == 0
+  rescue
+    @pid = nil
+  end
+end
+
+def kill(code)
+  Process.kill(code, @pid)
+  true
+rescue
+  false
+end
+
+def running?
+  @pid && kill(0)
+end
+
+def dead?
+  @pid && !kill(0)
+end
+
+def start
+  if running?
+    puts "already running (#{@pid})"
+    exit
+  end
+
+  if dead?
+    clean_pid
+  end
+
+  puts "starting #{NAME}"
+  system("#{DAEMON} -c #{CONFIGFILE} #{START_FLAGS}")
+end
+
+def stop
+  if not running?
+    puts "not running"
+    exit
+  end
+
+	system("god quit")
+end
+
+def restart
+	if running?
+		stop
+		read_pid
+	end
+	start
+end
+
+def force_reload
+	if running?
+	  restart
+	end
+end
+
+def clean_pid
+  File.delete(PIDFILE)
+end
+
+read_pid
+case ARGV[0]
+when 'start'
+  start
+when 'stop'
+  stop
+when 'restart'
+  if not running?
+    start
+  else
+    restart
+  end
+when 'force-reload'
+  force_reload
+when 'status'
+  if running?
+    puts "running (#{@pid})"
+  elsif dead?
+    puts "dead (#{@pid})"
+    exit!(1)
+  else
+    puts "not running"
+    exit!(3)
+  end
+else
+  puts "Usage: #{SCRIPTNAME} start|stop|restart|force-reload|status"
+end
+
+exit
diff -pruN 0.7.13-3/InstalledFiles 0.7.18-1/InstalledFiles
--- 0.7.13-3/InstalledFiles	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/InstalledFiles	2010-02-21 14:02:05.000000000 +0000
@@ -0,0 +1,53 @@
+/usr/bin/god
+/usr/lib/ruby/1.8/god.rb
+/usr/lib/ruby/1.8/god/sugar.rb
+/usr/lib/ruby/1.8/god/metric.rb
+/usr/lib/ruby/1.8/god/task.rb
+/usr/lib/ruby/1.8/god/simple_logger.rb
+/usr/lib/ruby/1.8/god/behavior.rb
+/usr/lib/ruby/1.8/god/diagnostics.rb
+/usr/lib/ruby/1.8/god/watch.rb
+/usr/lib/ruby/1.8/god/logger.rb
+/usr/lib/ruby/1.8/god/configurable.rb
+/usr/lib/ruby/1.8/god/contact.rb
+/usr/lib/ruby/1.8/god/condition.rb
+/usr/lib/ruby/1.8/god/driver.rb
+/usr/lib/ruby/1.8/god/dependency_graph.rb
+/usr/lib/ruby/1.8/god/errors.rb
+/usr/lib/ruby/1.8/god/process.rb
+/usr/lib/ruby/1.8/god/event_handler.rb
+/usr/lib/ruby/1.8/god/registry.rb
+/usr/lib/ruby/1.8/god/socket.rb
+/usr/lib/ruby/1.8/god/trigger.rb
+/usr/lib/ruby/1.8/god/timeline.rb
+/usr/lib/ruby/1.8/god/event_handlers/kqueue_handler.rb
+/usr/lib/ruby/1.8/god/event_handlers/netlink_handler.rb
+/usr/lib/ruby/1.8/god/event_handlers/dummy_handler.rb
+/usr/lib/ruby/1.8/god/system/slash_proc_poller.rb
+/usr/lib/ruby/1.8/god/system/process.rb
+/usr/lib/ruby/1.8/god/system/portable_poller.rb
+/usr/lib/ruby/1.8/god/behaviors/notify_when_flapping.rb
+/usr/lib/ruby/1.8/god/behaviors/clean_pid_file.rb
+/usr/lib/ruby/1.8/god/behaviors/clean_unix_socket.rb
+/usr/lib/ruby/1.8/god/contacts/twitter.rb
+/usr/lib/ruby/1.8/god/contacts/jabber.rb
+/usr/lib/ruby/1.8/god/contacts/campfire.rb
+/usr/lib/ruby/1.8/god/contacts/webhook.rb
+/usr/lib/ruby/1.8/god/contacts/email.rb
+/usr/lib/ruby/1.8/god/cli/command.rb
+/usr/lib/ruby/1.8/god/cli/run.rb
+/usr/lib/ruby/1.8/god/cli/version.rb
+/usr/lib/ruby/1.8/god/conditions/process_exits.rb
+/usr/lib/ruby/1.8/god/conditions/lambda.rb
+/usr/lib/ruby/1.8/god/conditions/always.rb
+/usr/lib/ruby/1.8/god/conditions/memory_usage.rb
+/usr/lib/ruby/1.8/god/conditions/complex.rb
+/usr/lib/ruby/1.8/god/conditions/file_mtime.rb
+/usr/lib/ruby/1.8/god/conditions/process_running.rb
+/usr/lib/ruby/1.8/god/conditions/tries.rb
+/usr/lib/ruby/1.8/god/conditions/degrading_lambda.rb
+/usr/lib/ruby/1.8/god/conditions/flapping.rb
+/usr/lib/ruby/1.8/god/conditions/cpu_usage.rb
+/usr/lib/ruby/1.8/god/conditions/http_response_code.rb
+/usr/lib/ruby/1.8/god/conditions/disk_usage.rb
+/usr/lib/ruby/1.8/i486-linux/netlink_handler_ext.so
Binary files 0.7.13-3/lib/god/cli/._command.rb and 0.7.18-1/lib/god/cli/._command.rb differ
diff -pruN 0.7.13-3/lib/god/cli/command.rb 0.7.18-1/lib/god/cli/command.rb
--- 0.7.13-3/lib/god/cli/command.rb	2008-11-14 04:55:51.000000000 +0000
+++ 0.7.18-1/lib/god/cli/command.rb	2010-02-21 14:01:37.000000000 +0000
@@ -117,11 +117,12 @@ module God
             exit!
           end
           
+          puts "Please wait..."
           t = Time.at(0)
           loop do
             print @server.running_log(name, t)
             t = Time.now
-            sleep 1
+            sleep 0.25
           end
         rescue God::NoSuchWatchError
           puts "No such watch"
Binary files 0.7.13-3/lib/god/cli/._run.rb and 0.7.18-1/lib/god/cli/._run.rb differ
diff -pruN 0.7.13-3/lib/god/cli/run.rb 0.7.18-1/lib/god/cli/run.rb
--- 0.7.13-3/lib/god/cli/run.rb	2008-11-21 22:54:32.000000000 +0000
+++ 0.7.18-1/lib/god/cli/run.rb	2010-02-21 14:01:37.000000000 +0000
@@ -58,7 +58,7 @@ module God
         end
         
         if @options[:config]
-          unless File.exist?(@options[:config])
+          if !@options[:config].include?('*') && !File.exist?(@options[:config])
             abort "File not found: #{@options[:config]}"
           end
           
@@ -142,23 +142,19 @@ module God
       end
       
       def load_config(config)
-        if File.directory? config
-          files_loaded = false
-          Dir[File.expand_path('**/*.god', config)].each do |god_file|
-            files_loaded ||= load_god_file(File.expand_path(god_file))
-          end
-          unless files_loaded
-            abort "No files could be loaded"
-          end
-        else
-          unless load_god_file(File.expand_path(config))
-            abort "File could not be loaded"
+        files = File.directory?(config) ? Dir['**/*.god'] : Dir[config]
+        abort "No files could be found" if files.empty?
+        files.each do |god_file|
+          unless load_god_file(god_file)
+            abort "File '#{god_file}' could not be loaded"
           end
         end
       end
       
       def load_god_file(god_file)
+        applog(nil, :info, "Loading #{god_file}")
         load File.expand_path(god_file)
+        true
       rescue Exception => e
         if e.instance_of?(SystemExit)
           raise
@@ -166,7 +162,7 @@ module God
           puts "There was an error in #{god_file}"
           puts "\t" + e.message
           puts "\t" + e.backtrace.join("\n\t")
-          return false
+          false
         end
       end
       
diff -pruN 0.7.13-3/lib/god/cli/version.rb 0.7.18-1/lib/god/cli/version.rb
--- 0.7.13-3/lib/god/cli/version.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/lib/god/cli/version.rb	2010-02-21 14:01:37.000000000 +0000
@@ -6,12 +6,12 @@ module God
         require 'god'
     
         # print version
-        puts "Version #{God::VERSION}"
+        puts "Version #{God.version}"
         exit
       end
       
       def self.version_extended
-        puts "Version: #{God::VERSION}"
+        puts "Version: #{God.version}"
         puts "Polls: enabled"
         puts "Events: " + God::EventHandler.event_system
     
diff -pruN 0.7.13-3/lib/god/conditions/disk_usage.rb 0.7.18-1/lib/god/conditions/disk_usage.rb
--- 0.7.13-3/lib/god/conditions/disk_usage.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/lib/god/conditions/disk_usage.rb	2010-02-21 14:01:37.000000000 +0000
@@ -18,10 +18,15 @@ module God
       end
       
       def test
-        usage = `df | grep -i " #{self.mount_point}$" | awk '{print $5}' | sed 's/%//'`
-        usage.to_i > self.above
+        usage = `df -P | grep -i " #{self.mount_point}$" | awk '{print $5}' | sed 's/%//'`
+        if usage.to_i > self.above
+          self.info = "disk space out of bounds"
+          return true
+        else
+          self.info = "disk space ok"
+          return false
+        end
       end
     end
-    
   end
-end
+end
\ No newline at end of file
Binary files 0.7.13-3/lib/god/conditions/._file_mtime.rb and 0.7.18-1/lib/god/conditions/._file_mtime.rb differ
Binary files 0.7.13-3/lib/god/conditions/._flapping.rb and 0.7.18-1/lib/god/conditions/._flapping.rb differ
Binary files 0.7.13-3/lib/god/._configurable.rb and 0.7.18-1/lib/god/._configurable.rb differ
Binary files 0.7.13-3/lib/god/contacts/._campfire.rb and 0.7.18-1/lib/god/contacts/._campfire.rb differ
diff -pruN 0.7.13-3/lib/god/contacts/campfire.rb 0.7.18-1/lib/god/contacts/campfire.rb
--- 0.7.13-3/lib/god/contacts/campfire.rb	2008-11-14 01:16:53.000000000 +0000
+++ 0.7.18-1/lib/god/contacts/campfire.rb	2010-02-21 14:01:37.000000000 +0000
@@ -39,7 +39,8 @@ module God
       self.server_settings = {:subdomain => '',
                               :user_name => '',
                               :password => '',
-                              :room => ''}
+                              :room => '',
+                              :ssl => false}
 
       self.format = lambda do |message, host|
         <<-EOF
@@ -70,7 +71,7 @@ module God
         unless @room
           applog(nil,:debug, "initializing campfire connection using credentials: #{Campfire.server_settings.inspect}")
 
-          campfire = Tinder::Campfire.new Campfire.server_settings[:subdomain] 
+          campfire = Tinder::Campfire.new Campfire.server_settings[:subdomain], :ssl => Campfire.server_settings[:ssl]
           campfire.login Campfire.server_settings[:user_name], Campfire.server_settings[:password]
           @room = campfire.find_room_by_name(Campfire.server_settings[:room])
         end
diff -pruN 0.7.13-3/lib/god/contacts/email.rb 0.7.18-1/lib/god/contacts/email.rb
--- 0.7.13-3/lib/god/contacts/email.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/lib/god/contacts/email.rb	2010-02-21 14:01:37.000000000 +0000
@@ -30,7 +30,7 @@ From: god <#{self.message_settings[:from
 To: #{name} <#{email}>
 Subject: [god] #{message}
 Date: #{Time.now.httpdate}
-Message-Id: <unique.message.id.string@example.com>
+Message-Id: <#{rand(1000000000).to_s(36)}.#{$$}.#{self.message_settings[:from]}>
 
 Message: #{message}
 Host: #{host}
Binary files 0.7.13-3/lib/god/contacts/._jabber.rb and 0.7.18-1/lib/god/contacts/._jabber.rb differ
diff -pruN 0.7.13-3/lib/god/contacts/jabber.rb 0.7.18-1/lib/god/contacts/jabber.rb
--- 0.7.13-3/lib/god/contacts/jabber.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/lib/god/contacts/jabber.rb	2010-02-21 14:01:37.000000000 +0000
@@ -19,7 +19,7 @@ module God
   module Contacts
     class Jabber < Contact      
       class << self
-        attr_accessor :settings, :format
+        attr_accessor :settings, :format, :client
       end
       
       self.format = lambda do |message, priority, category, host|
@@ -37,29 +37,91 @@ module God
       end
       
       def notify(message, time, priority, category, host)
+        connect
+        
+        body = Jabber.format.call message, priority, category, host
+        
+        message = XMPP4R::Message::new self.jabber_id, body
+        message.set_type :normal
+        message.set_id '1'
+        message.set_subject 'God'
+        
+        self.send!(message)
+        
+        self.info = "sent jabber message to #{self.jabber_id}"
+      rescue => e
+        puts e.message
+        puts e.backtrace.join("\n")
+        
+        self.info = "failed to send jabber message to #{self.jabber_id}: #{e.message}"
+      end
+      
+      def send!(msg)
+        attempts = 0
         begin
-          jabber_id = XMPP4R::JID::new "#{Jabber.settings[:jabber_id]}/God"
-          jabber_client = XMPP4R::Client::new jabber_id
-          jabber_client.connect
-          jabber_client.auth Jabber.settings[:password]
-
-          body = Jabber.format.call message, priority, category, host
-          
-          message = XMPP4R::Message::new self.jabber_id, body
-          message.set_type :normal
-          message.set_id '1'
-          message.set_subject 'God'
-          jabber_client.send message
+          attempts += 1
+          client.send(msg)
+        rescue Errno::EPIPE, IOError => e
+          sleep 1
+          disconnect!
+          reconnect!
+          retry unless attempts > 3
+          raise e
+        rescue Errno::ECONNRESET => e
+          sleep (attempts^2) * 60 + 60
+          disconnect!
+          reconnect!
+          retry unless attempts > 3
+          raise e
+        end
+      end
+      
+      def connect
+        connect! unless connected?
+      end
+      
+      def connected?
+        connected = client.respond_to?(:is_connected?) && client.is_connected?
+        return connected
+      end
+      
+      def connect!
+        disconnect! if connected?
 
-          self.info = "sent jabber message to #{self.jabber_id}"
-        rescue => e
-          puts e.message
-          puts e.backtrace.join("\n")
-          
-          self.info = "failed to send jabber message to #{self.jabber_id}: #{e.message}"
+        @connect_mutex ||= Mutex.new
+        # don't try to connect if another thread is already connecting.
+        return if @connect_mutex.locked?
+        @connect_mutex.lock
+        
+        jabber_id = XMPP4R::JID::new "#{Jabber.settings[:jabber_id]}/God"
+        jabber_client = XMPP4R::Client::new jabber_id
+        jabber_client.connect Jabber.settings[:host]
+        jabber_client.auth Jabber.settings[:password]
+        self.client = jabber_client
+        
+        @connect_mutex.unlock
+      end
+      
+      def disconnect!
+        if client.respond_to?(:is_connected?) && client.is_connected?
+          begin
+            client.close
+          rescue Errno::EPIPE, IOError => e
+            self.info "Failed to disconnect: #{e}"
+            nil
+          end
         end
+        client = nil
+      end
+
+      def client
+        Jabber.client
+      end
+
+      def client=(jc)
+        Jabber.client = jc
       end
+
     end
-    
   end
 end
\ No newline at end of file
Binary files 0.7.13-3/lib/god/contacts/._twitter.rb and 0.7.18-1/lib/god/contacts/._twitter.rb differ
Binary files 0.7.13-3/lib/god/contacts/._webhook.rb and 0.7.18-1/lib/god/contacts/._webhook.rb differ
Binary files 0.7.13-3/lib/god/._driver.rb and 0.7.18-1/lib/god/._driver.rb differ
diff -pruN 0.7.13-3/lib/god/driver.rb 0.7.18-1/lib/god/driver.rb
--- 0.7.13-3/lib/god/driver.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/lib/god/driver.rb	2010-02-21 14:01:37.000000000 +0000
@@ -90,7 +90,8 @@ module God
             Thread.stop
           else
             Thread.critical = false
-            sleep @events.first.at - Time.now
+            delay = @events.first.at - Time.now
+            sleep delay if delay > 0
             Thread.critical = true
           end
         end
@@ -166,7 +167,7 @@ module God
           rescue ThreadError => e
             # queue is empty
             break
-          rescue Exception => e
+          rescue Object => e
             message = format("Unhandled exception in driver loop - (%s): %s\n%s",
                              e.class, e.message, e.backtrace.join("\n"))
             applog(nil, :fatal, message)
Binary files 0.7.13-3/lib/god/._event_handler.rb and 0.7.18-1/lib/god/._event_handler.rb differ
Binary files 0.7.13-3/lib/god/._logger.rb and 0.7.18-1/lib/god/._logger.rb differ
diff -pruN 0.7.13-3/lib/god/logger.rb 0.7.18-1/lib/god/logger.rb
--- 0.7.13-3/lib/god/logger.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/lib/god/logger.rb	2010-02-21 14:01:37.000000000 +0000
@@ -16,11 +16,12 @@ module God
     self.syslog ||= true
     
     # Instantiate a new Logger object
-    def initialize
-      super($stdout)
+    def initialize(io = $stdout)
+      super(io)
       self.logs = {}
       @mutex = Mutex.new
       @capture = nil
+      @spool = Time.now - 10
       @templogio = StringIO.new
       @templog = SimpleLogger.new(@templogio)
       @templog.level = Logger::INFO
@@ -64,7 +65,9 @@ module God
       @templog.send(level, text % [])
       @mutex.synchronize do
         @capture.puts(@templogio.string.dup) if @capture
-        self.logs[watch.name] << [Time.now, @templogio.string.dup] if watch
+        if watch && (Time.now - @spool < 2)
+          self.logs[watch.name] << [Time.now, @templogio.string.dup]
+        end
       end
       
       # send to regular logger
@@ -85,6 +88,7 @@ module God
       
       # get and join lines since given time
       @mutex.synchronize do
+        @spool = Time.now
         self.logs[watch_name].select do |x|
           x.first > since
         end.map do |x|
Binary files 0.7.13-3/lib/god/._process.rb and 0.7.18-1/lib/god/._process.rb differ
diff -pruN 0.7.13-3/lib/god/process.rb 0.7.18-1/lib/god/process.rb
--- 0.7.13-3/lib/god/process.rb	2008-11-14 05:04:20.000000000 +0000
+++ 0.7.18-1/lib/god/process.rb	2010-02-21 14:01:37.000000000 +0000
@@ -2,7 +2,8 @@ module God
   class Process
     WRITES_PID = [:start, :restart]
     
-    attr_accessor :name, :uid, :gid, :log, :log_cmd, :start, :stop, :restart, :unix_socket, :chroot, :env
+    attr_accessor :name, :uid, :gid, :log, :log_cmd, :start, :stop, :restart,
+                  :unix_socket, :chroot, :env, :dir
     
     def initialize
       self.log = '/dev/null'
@@ -25,13 +26,17 @@ module God
     
     def file_writable?(file)
       pid = fork do
-        uid_num = Etc.getpwnam(self.uid).uid if self.uid
-        gid_num = Etc.getgrnam(self.gid).gid if self.gid
+        begin
+          uid_num = Etc.getpwnam(self.uid).uid if self.uid
+          gid_num = Etc.getgrnam(self.gid).gid if self.gid
 
-        ::Dir.chroot(self.chroot) if self.chroot
-        ::Process.groups = [gid_num] if self.gid
-        ::Process::Sys.setgid(gid_num) if self.gid
-        ::Process::Sys.setuid(uid_num) if self.uid
+          ::Dir.chroot(self.chroot) if self.chroot
+          ::Process.groups = [gid_num] if self.gid
+          ::Process::Sys.setgid(gid_num) if self.gid
+          ::Process::Sys.setuid(uid_num) if self.uid
+        rescue ArgumentError, Errno::EPERM, Errno::ENOENT
+          exit(1)
+        end
 
         File.writable?(file_in_chroot(file)) ? exit(0) : exit(1)
       end
@@ -52,12 +57,6 @@ module God
         applog(self, :error, "No start command was specified")
       end
       
-      # self-daemonizing processes must specify a stop command
-      if !@tracking_pid && self.stop.nil?
-        valid = false
-        applog(self, :error, "No stop command was specified")
-      end
-      
       # uid must exist if specified
       if self.uid
         begin
@@ -78,6 +77,17 @@ module God
         end
       end
       
+      # dir must exist and be a directory if specified
+      if self.dir
+        if !File.exist?(self.dir)
+          valid = false
+          applog(self, :error, "Specified directory '#{self.dir}' does not exist")
+        elsif !File.directory?(self.dir)
+          valid = false
+          applog(self, :error, "Specified directory '#{self.dir}' is not a directory")
+        end
+      end
+      
       # pid dir must exist if specified
       if !@tracking_pid && !File.exist?(File.dirname(self.pid_file))
         valid = false
@@ -113,12 +123,12 @@ module God
       if self.chroot
         if !File.directory?(self.chroot)
           valid = false
-          LOG.log(self, :error, "CHROOT directory '#{self.chroot}' does not exist")
+          applog(self, :error, "CHROOT directory '#{self.chroot}' does not exist")
         end
 
         if !File.exist?(File.join(self.chroot, '/dev/null'))
           valid = false
-          LOG.log(self, :error, "CHROOT directory '#{self.chroot}' does not contain '/dev/null'")
+          applog(self, :error, "CHROOT directory '#{self.chroot}' does not contain '/dev/null'")
         end
       end
       
@@ -281,7 +291,8 @@ module God
         ::Process.groups = [gid_num] if self.gid
         ::Process::Sys.setgid(gid_num) if self.gid
         ::Process::Sys.setuid(uid_num) if self.uid
-        Dir.chdir "/"
+        self.dir ||= '/'
+        Dir.chdir self.dir
         $0 = command
         STDIN.reopen "/dev/null"
         if self.log_cmd
Binary files 0.7.13-3/lib/god/._registry.rb and 0.7.18-1/lib/god/._registry.rb differ
Binary files 0.7.13-3/lib/god/._socket.rb and 0.7.18-1/lib/god/._socket.rb differ
Binary files 0.7.13-3/lib/god/system/._portable_poller.rb and 0.7.18-1/lib/god/system/._portable_poller.rb differ
Binary files 0.7.13-3/lib/god/system/._process.rb and 0.7.18-1/lib/god/system/._process.rb differ
Binary files 0.7.13-3/lib/god/._task.rb and 0.7.18-1/lib/god/._task.rb differ
diff -pruN 0.7.13-3/lib/god/task.rb 0.7.18-1/lib/god/task.rb
--- 0.7.13-3/lib/god/task.rb	2008-12-10 20:37:53.000000000 +0000
+++ 0.7.18-1/lib/god/task.rb	2010-02-21 14:01:37.000000000 +0000
@@ -318,7 +318,15 @@ module God
       metric = self.directory[condition]
       
       # run the test
-      result = condition.test
+      begin
+        result = condition.test
+      rescue Object => e
+        cname = condition.class.to_s.split('::').last
+        message = format("Unhandled exception in %s condition - (%s): %s\n%s",
+                         cname, e.class, e.message, e.backtrace.join("\n"))
+        applog(self, :error, message)
+        result = false
+      end
       
       # log
       messages = self.log_line(self, metric, condition, result)
@@ -479,13 +487,17 @@ module God
       # notify each contact
       resolved_contacts.each do |c|
         host = `hostname`.chomp rescue 'none'
-        c.notify(message, Time.now, spec[:priority], spec[:category], host)
-        
-        msg = "#{condition.watch.name} #{c.info ? c.info : "notification sent for contact: #{c.name}"} (#{c.base_name})"
-        
-        applog(condition.watch, :info, msg % [])
+        begin
+          c.notify(message, Time.now, spec[:priority], spec[:category], host)
+          msg = "#{condition.watch.name} #{c.info ? c.info : "notification sent for contact: #{c.name}"} (#{c.base_name})"
+          applog(condition.watch, :info, msg % [])
+        rescue Exception => e
+          applog(condition.watch, :error, "#{e.message} #{e.backtrace}")
+          msg = "#{condition.watch.name} Failed to deliver notification for contact: #{c.name} (#{c.base_name})"
+          applog(condition.watch, :error, msg % [])
+        end
       end
     end
   end
   
-end
\ No newline at end of file
+end
Binary files 0.7.13-3/lib/god/._timeline.rb and 0.7.18-1/lib/god/._timeline.rb differ
Binary files 0.7.13-3/lib/god/._trigger.rb and 0.7.18-1/lib/god/._trigger.rb differ
Binary files 0.7.13-3/lib/god/._watch.rb and 0.7.18-1/lib/god/._watch.rb differ
diff -pruN 0.7.13-3/lib/god/watch.rb 0.7.18-1/lib/god/watch.rb
--- 0.7.13-3/lib/god/watch.rb	2008-11-14 04:48:00.000000000 +0000
+++ 0.7.18-1/lib/god/watch.rb	2010-02-21 14:01:37.000000000 +0000
@@ -11,8 +11,8 @@ module God
     attr_accessor :grace, :start_grace, :stop_grace, :restart_grace
     
     extend Forwardable
-    def_delegators :@process, :name, :uid, :gid, :start, :stop, :restart,
-                              :name=, :uid=, :gid=, :start=, :stop=, :restart=,
+    def_delegators :@process, :name, :uid, :gid, :start, :stop, :restart, :dir,
+                              :name=, :uid=, :gid=, :start=, :stop=, :restart=, :dir=,
                               :pid_file, :pid_file=, :log, :log=, :log_cmd, :log_cmd=, :alive?, :pid,
                               :unix_socket, :unix_socket=, :chroot, :chroot=, :env, :env=, :signal
     # 
Binary files 0.7.13-3/lib/._god.rb and 0.7.18-1/lib/._god.rb differ
diff -pruN 0.7.13-3/lib/god.rb 0.7.18-1/lib/god.rb
--- 0.7.13-3/lib/god.rb	2009-05-04 08:55:01.000000000 +0100
+++ 0.7.18-1/lib/god.rb	2010-02-21 14:01:37.000000000 +0000
@@ -148,8 +148,6 @@ class Module
 end
 
 module God
-  VERSION = '0.7.13'
-  
   LOG_BUFFER_SIZE_DEFAULT = 100
   PID_FILE_DIRECTORY_DEFAULTS = ['/var/run/god', '~/.god/pids']
   DRB_PORT_DEFAULT = 17165
@@ -626,6 +624,11 @@ module God
     self.main.join
   end
   
+  def self.version
+    yml = YAML.load(File.read(File.join(File.dirname(__FILE__), *%w[.. VERSION.yml])))
+    "#{yml[:major]}.#{yml[:minor]}.#{yml[:patch]}"
+  end
+  
   # To be called on program exit to start god
   #
   # Returns nothing
@@ -654,7 +657,7 @@ module God
     
     list.select do |item|
       item =~ Regexp.new(regex)
-    end
+    end.sort_by { |x| x.size }
   end
 end
 
Binary files 0.7.13-3/._Manifest.txt and 0.7.18-1/._Manifest.txt differ
diff -pruN 0.7.13-3/Manifest.txt 0.7.18-1/Manifest.txt
--- 0.7.13-3/Manifest.txt	2008-11-14 01:59:36.000000000 +0000
+++ 0.7.18-1/Manifest.txt	1970-01-01 01:00:00.000000000 +0100
@@ -1,114 +0,0 @@
-History.txt
-Manifest.txt
-README.txt
-Rakefile
-bin/god
-examples/events.god
-examples/gravatar.god
-examples/single.god
-ext/god/extconf.rb
-ext/god/kqueue_handler.c
-ext/god/netlink_handler.c
-init/god
-lib/god.rb
-lib/god/behavior.rb
-lib/god/behaviors/clean_pid_file.rb
-lib/god/behaviors/clean_unix_socket.rb
-lib/god/behaviors/notify_when_flapping.rb
-lib/god/cli/command.rb
-lib/god/cli/run.rb
-lib/god/cli/version.rb
-lib/god/condition.rb
-lib/god/conditions/always.rb
-lib/god/conditions/complex.rb
-lib/god/conditions/cpu_usage.rb
-lib/god/conditions/degrading_lambda.rb
-lib/god/conditions/disk_usage.rb
-lib/god/conditions/file_mtime.rb
-lib/god/conditions/flapping.rb
-lib/god/conditions/http_response_code.rb
-lib/god/conditions/lambda.rb
-lib/god/conditions/memory_usage.rb
-lib/god/conditions/process_exits.rb
-lib/god/conditions/process_running.rb
-lib/god/conditions/tries.rb
-lib/god/configurable.rb
-lib/god/contact.rb
-lib/god/contacts/campfire.rb
-lib/god/contacts/email.rb
-lib/god/contacts/jabber.rb
-lib/god/contacts/twitter.rb
-lib/god/contacts/webhook.rb
-lib/god/dependency_graph.rb
-lib/god/diagnostics.rb
-lib/god/driver.rb
-lib/god/errors.rb
-lib/god/event_handler.rb
-lib/god/event_handlers/dummy_handler.rb
-lib/god/event_handlers/kqueue_handler.rb
-lib/god/event_handlers/netlink_handler.rb
-lib/god/logger.rb
-lib/god/metric.rb
-lib/god/process.rb
-lib/god/registry.rb
-lib/god/simple_logger.rb
-lib/god/socket.rb
-lib/god/sugar.rb
-lib/god/system/portable_poller.rb
-lib/god/system/process.rb
-lib/god/system/slash_proc_poller.rb
-lib/god/task.rb
-lib/god/timeline.rb
-lib/god/trigger.rb
-lib/god/watch.rb
-test/configs/child_events/child_events.god
-test/configs/child_events/simple_server.rb
-test/configs/child_polls/child_polls.god
-test/configs/child_polls/simple_server.rb
-test/configs/complex/complex.god
-test/configs/complex/simple_server.rb
-test/configs/contact/contact.god
-test/configs/contact/simple_server.rb
-test/configs/daemon_events/daemon_events.god
-test/configs/daemon_events/simple_server.rb
-test/configs/daemon_events/simple_server_stop.rb
-test/configs/daemon_polls/daemon_polls.god
-test/configs/daemon_polls/simple_server.rb
-test/configs/degrading_lambda/degrading_lambda.god
-test/configs/degrading_lambda/tcp_server.rb
-test/configs/matias/matias.god
-test/configs/real.rb
-test/configs/running_load/running_load.god
-test/configs/stress/simple_server.rb
-test/configs/stress/stress.god
-test/configs/task/logs/.placeholder
-test/configs/task/task.god
-test/configs/test.rb
-test/helper.rb
-test/suite.rb
-test/test_behavior.rb
-test/test_campfire.rb
-test/test_condition.rb
-test/test_conditions_disk_usage.rb
-test/test_conditions_http_response_code.rb
-test/test_conditions_process_running.rb
-test/test_conditions_tries.rb
-test/test_contact.rb
-test/test_dependency_graph.rb
-test/test_driver.rb
-test/test_email.rb
-test/test_event_handler.rb
-test/test_god.rb
-test/test_handlers_kqueue_handler.rb
-test/test_logger.rb
-test/test_metric.rb
-test/test_process.rb
-test/test_registry.rb
-test/test_socket.rb
-test/test_sugar.rb
-test/test_system_portable_poller.rb
-test/test_system_process.rb
-test/test_task.rb
-test/test_timeline.rb
-test/test_trigger.rb
-test/test_watch.rb
diff -pruN 0.7.13-3/Rakefile 0.7.18-1/Rakefile
--- 0.7.13-3/Rakefile	2009-05-04 08:55:17.000000000 +0100
+++ 0.7.18-1/Rakefile	2010-02-21 14:01:37.000000000 +0000
@@ -1,17 +1,35 @@
 require 'rubygems'
-require 'hoe'
+require 'rake'
 
-Hoe.new('god', '0.7.13') do |p|
-  p.rubyforge_name = 'god'
-  p.author = 'Tom Preston-Werner'
-  p.email = 'tom@rubyisawesome.com'
-  p.url = 'http://god.rubyforge.org/'
-  p.summary = 'Like monit, only awesome'
-  p.description = "God is an easy to configure, easy to extend monitoring framework written in Ruby."
-  p.changes = p.paragraphs_of('History.txt', 0..1).join("\n\n")
-  p.spec_extras = {:extensions => ['ext/god/extconf.rb']}
+begin
+  require 'jeweler'
+  Jeweler::Tasks.new do |gem|
+    gem.name = "god"
+    gem.rubyforge_project = "god"
+    gem.summary = 'Like monit, only awesome'
+    gem.description = "God is an easy to configure, easy to extend monitoring framework written in Ruby."
+    gem.email = "tom@mojombo.com"
+    gem.homepage = "http://god.rubyforge.org/"
+    gem.authors = ["Tom Preston-Werner"]
+    gem.require_paths = ["lib", "ext"]
+    gem.files.include("ext")
+    gem.extensions << 'ext/god/extconf.rb'
+    # gem is a Gem::Specification... see http://www.rubygems.org/read/chapter/20 for additional settings
+  end
+
+rescue LoadError
+  puts "Jeweler (or a dependency) not available. Install it with: sudo gem install jeweler"
+end
+
+require 'rake/testtask'
+Rake::TestTask.new(:test) do |test|
+  test.libs << 'lib' << 'test'
+  test.pattern = 'test/**/test_*.rb'
+  test.verbose = true
 end
 
+task :default => :test
+
 desc "Open an irb session preloaded with this library"
 task :console do
   sh "irb -rubygems -r ./lib/god.rb"
@@ -32,4 +50,19 @@ task :coverage do
   `rm -fr coverage`
   `rcov test/test_*.rb`
   `open coverage/index.html`
+end
+
+require 'rake/rdoctask'
+Rake::RDocTask.new do |rdoc|
+  if File.exist?('VERSION.yml')
+    config = YAML.load(File.read('VERSION.yml'))
+    version = "#{config[:major]}.#{config[:minor]}.#{config[:patch]}"
+  else
+    version = ""
+  end
+
+  rdoc.rdoc_dir = 'rdoc'
+  rdoc.title = "god #{version}"
+  rdoc.rdoc_files.include('README*')
+  rdoc.rdoc_files.include('lib/**/*.rb')
 end
\ No newline at end of file
Binary files 0.7.13-3/._Rakefile and 0.7.18-1/._Rakefile differ
Binary files 0.7.13-3/._README.txt and 0.7.18-1/._README.txt differ
Binary files 0.7.13-3/site/images/banner.jpg and 0.7.18-1/site/images/banner.jpg differ
Binary files 0.7.13-3/site/images/bg.gif and 0.7.18-1/site/images/bg.gif differ
Binary files 0.7.13-3/site/images/bg_grey.gif and 0.7.18-1/site/images/bg_grey.gif differ
Binary files 0.7.13-3/site/images/bullet.jpg and 0.7.18-1/site/images/bullet.jpg differ
Binary files 0.7.13-3/site/images/corner_green.gif and 0.7.18-1/site/images/corner_green.gif differ
Binary files 0.7.13-3/site/images/corner_green.psd and 0.7.18-1/site/images/corner_green.psd differ
Binary files 0.7.13-3/site/images/corner_pink.gif and 0.7.18-1/site/images/corner_pink.gif differ
Binary files 0.7.13-3/site/images/god_logo.png and 0.7.18-1/site/images/god_logo.png differ
Binary files 0.7.13-3/site/images/header_bg.gif and 0.7.18-1/site/images/header_bg.gif differ
Binary files 0.7.13-3/site/images/header_bg.jpg and 0.7.18-1/site/images/header_bg.jpg differ
Binary files 0.7.13-3/site/images/red_dot.gif and 0.7.18-1/site/images/red_dot.gif differ
Binary files 0.7.13-3/site/images/top_bg.gif and 0.7.18-1/site/images/top_bg.gif differ
diff -pruN 0.7.13-3/site/index.html 0.7.18-1/site/index.html
--- 0.7.13-3/site/index.html	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/site/index.html	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,563 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html xmlns="http://www.w3.org/1999/xhtml">
+<head>
+  <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
+  <title>God - A Process Monitoring Framework in Ruby</title>
+  <link href="stylesheets/layout.css" rel="stylesheet" type="text/css" />
+  <script type="text/javascript" src="javascripts/code_highlighter.js"></script>
+  <script type="text/javascript" src="javascripts/ruby.js"></script>
+</head>
+
+<body id="page_home">
+
+<div id="mothership">
+
+</div>
+<div id="content">
+	<div class="banner">
+
+	</div>
+
+  <!-- <div id="menu">
+    <div class="dots"></div>
+    <ul>
+      <li class="menu_home"><a href="/">Home</a></li>
+      <li class="menu_contact"><a href="mailto:tom@projectmothership.com">Contact</a></li>
+    </ul>
+    <div class="dots"></div>
+  </div> -->
+
+	<div class="columnleft">
+		<h1>A Better Way to Monitor</h1>
+		<p>God is an easy to configure, easy to extend monitoring framework written in Ruby.</p>
+		<p>Keeping your server processes and tasks running should be a simple part of your deployment process. God aims to be the simplest, most powerful monitoring application available.</p>
+		<p style="text-align: right">Tom Preston-Werner<br /><a href="mailto:tom@mojombo.com">tom@mojombo.com</a></p>
+		<p style="text-align: right">Google Group: <a href="http://groups.google.com/group/god-rb">http://groups.google.com/group/god-rb</a></p>
+	</div>
+
+	<div class="columnright">
+	<h1>Features</h1>
+	<ul class="features">
+		<li>Config file is written in Ruby</li>
+		<li>Easily write your own custom conditions in Ruby</li>
+		<li>Supports both poll and event based conditions</li>
+		<li>Different poll conditions can have different intervals</li>
+		<li>Integrated notification system (write your own too!)</li>
+		<li>Easily control non-daemonizing scripts</li>
+	</ul>
+	</div>
+	
+	<h1>Installation (v 0.7.17)</h1>
+	<p>The best way to get god is via rubygems:</p>
+	<pre>$ sudo gem install god</pre>
+	
+	<h1>Contribute</h1>
+	<p>God is open source and accepting pull requests via GitHub!</p>
+	<p><a href="http://github.com/mojombo/god">http://github.com/mojombo/god</a></p>
+	
+	<h1>Requirements</h1>
+	
+	<p>God currently only works on <b>Linux (kernel 2.6.15+), BSD,</b> and <b>Darwin</b> systems. No support for Windows is planned. Event based conditions on Linux systems require the <code>cn</code> (connector) kernel module loaded or compiled in to the kernel and god must be run as root.</p>
+	
+	<p>The following systems have been tested. Help us test it on others!</p>
+	
+	<ul>
+	  <li>Darwin 10.4.10</li>
+	  <li>RedHat Fedora Core 6</li>
+	  <li>Ubuntu Dapper (no events)</li>
+	  <li>Ubuntu Feisty</li>
+	  <li>CentOS 4.5 (no events)</li>
+	</ul>
+
+	<h1>Config Files are Ruby Code!</h1>
+	<p>The easiest way to understand how god will make your life better is by looking at a sample config file. The following configuration file is what I once used at <a href="http://site.gravatar.com/">gravatar.com</a> to keep the mongrels running:</p>
+	
+<pre><code class="ruby"># run with: god -c /path/to/mongrel.god -D
+# 
+# This is the actual config file used to keep the mongrels of
+# gravatar.com running.
+
+RAILS_ROOT = "/Users/tom/dev/gravatar2"
+
+%w{8200 8201 8202}.each do |port|
+  God.watch do |w|
+    w.name = "gravatar2-mongrel-#{port}"
+    w.interval = 30.seconds # default      
+    w.start = "mongrel_rails start -c #{RAILS_ROOT} -p #{port} \
+      -P #{RAILS_ROOT}/log/mongrel.#{port}.pid  -d"
+    w.stop = "mongrel_rails stop -P #{RAILS_ROOT}/log/mongrel.#{port}.pid"
+    w.restart = "mongrel_rails restart -P #{RAILS_ROOT}/log/mongrel.#{port}.pid"
+    w.start_grace = 10.seconds
+    w.restart_grace = 10.seconds
+    w.pid_file = File.join(RAILS_ROOT, "log/mongrel.#{port}.pid")
+    
+    w.behavior(:clean_pid_file)
+
+    w.start_if do |start|
+      start.condition(:process_running) do |c|
+        c.interval = 5.seconds
+        c.running = false
+      end
+    end
+    
+    w.restart_if do |restart|
+      restart.condition(:memory_usage) do |c|
+        c.above = 150.megabytes
+        c.times = [3, 5] # 3 out of 5 intervals
+      end
+    
+      restart.condition(:cpu_usage) do |c|
+        c.above = 50.percent
+        c.times = 5
+      end
+    end
+    
+    # lifecycle
+    w.lifecycle do |on|
+      on.condition(:flapping) do |c|
+        c.to_state = [:start, :restart]
+        c.times = 5
+        c.within = 5.minute
+        c.transition = :unmonitored
+        c.retry_in = 10.minutes
+        c.retry_times = 5
+        c.retry_within = 2.hours
+      end
+    end
+  end
+end</code></pre>
+
+<p>That's a lot to take in at once, so I'll break it down by section and explain what's going on in each.</p>
+
+<pre><code class="ruby">RAILS_ROOT = "/var/www/gravatar2/current"</code></pre>
+
+<p>Here I've set a constant that is used throughout the file. Keeping the <code>RAILS_ROOT</code> value in a constant makes it easy to adapt this script to other applications. Because the config file is Ruby code, I can set whatever variables or constants I want that make the configuration more concise and easier to work with.</p>
+
+<pre><code class="ruby">%w{8200 8201 8202}.each do |port|
+  ...
+end</code></pre>
+
+<p>Because the config file is written in actual Ruby code, we can construct loops and do other intelligent things that are impossible in your every day, run of the mill config file. I need to watch three mongrels, so I simply loop over their port numbers, eliminating duplication and making my life a whole lot easier.</p>
+
+<pre><code class="ruby">  God.watch do |w|
+    w.name = "gravatar2-mongrel-#{port}"
+    w.interval = 30.seconds # default      
+    w.start = "mongrel_rails start -c #{RAILS_ROOT} -p #{port} \
+      -P #{RAILS_ROOT}/log/mongrel.#{port}.pid  -d"
+    w.stop = "mongrel_rails stop -P #{RAILS_ROOT}/log/mongrel.#{port}.pid"
+    w.restart = "mongrel_rails restart -P #{RAILS_ROOT}/log/mongrel.#{port}.pid"
+    w.start_grace = 10.seconds
+    w.restart_grace = 10.seconds
+    w.pid_file = File.join(RAILS_ROOT, "log/mongrel.#{port}.pid")
+        
+    ...
+  end</code></pre>
+
+<p>A <code>watch</code> represents a single process that has concrete start, stop, and/or restart operations. You can define as many watches as you like. In the example above, I've got some Rails instances running in Mongrels that I need to keep alive. Every watch must have a unique <code>name</code> so that it can be identified later on. The <code>interval</code> option sets the default poll interval (this can be overridden in each condition). The <code>start</code> and <code>stop</code> attributes specify the commands to start and stop the process. If no <code>restart</code> attribute is set, restart will be represented by a call to stop followed by a call to start. The optional <code>grace</code> attribute sets the amount of time following a start/stop/restart command to wait before resuming normal monitoring operations. To be more specific, I can set just <code>start_grace</code>, <code>stop_grace</code>, and/or <code>restart_grace</code>. If the process you're watching runs as a daemon (as mine does), you'll need to set the <code>pid_file</code> attribute.</p>
+
+<pre><code class="ruby">    w.behavior(:clean_pid_file)</code></pre>
+      
+<p>Behaviors allow you to execute additional commands around start/stop/restart commands. In our case, if the process dies it will leave a PID file behind. The next time a start command is issued, it will fail, complaining about the leftover PID file. We'd like the PID file cleaned up before a start command is issued. The built-in behavior <code>clean_pid_file</code> will do just that.</p>
+
+<pre><code class="ruby">    w.start_if do |start|
+      start.condition(:process_running) do |c|
+        c.interval = 5.seconds
+        c.running = false
+      end
+    end</code></pre>
+      
+<p>Watches contain conditions grouped by the action to execute should they return <code>true</code>. I start with a <code>start_if</code> block that contains a single condition. Conditions are specified by calling <code>condition</code> with an identifier, in this case 
+<code>:process_running</code>. Each condition can specify a poll interval that will override the default watch interval. In this case, I want to check that the process is still running every 5 seconds instead of the 30 second interval that other conditions will inherit. The ability to set condition specific poll intervals makes it possible to run critical tests (such as :process_running) more often than less critical tests (such as :memory_usage and :cpu_usage).</p>
+
+<pre><code class="ruby">    w.restart_if do |restart|
+      restart.condition(:memory_usage) do |c|
+        c.above = 150.megabytes
+        c.times = [3, 5] # 3 out of 5 intervals
+      end
+    
+      ...
+    end</code></pre>
+
+<p>Similar to <code>start_if</code> there is a <code>restart_if</code> command that groups conditions that should trigger a restart. The <code>memory_usage</code> condition will fail if the specified process is using too much memory. The maximum allowable amount of memory is specified with the <code>above</code> attribute (you can use the kilobytes, megabytes, or gigabytes helpers). The number of times the test needs to fail in order to trigger a restart is set with <code>times</code>. This can be either an integer or an array. An integer means it must fail that many times in a row while an array [x, y] means it must fail x times out of the last y tests.</p>
+
+<pre><code class="ruby">    w.restart_if do |restart|
+      ...
+    
+      restart.condition(:cpu_usage) do |c|
+        c.above = 50.percent
+        c.times = 5
+      end
+    end</code></pre>
+      
+<p>To keep an eye on CPU usage, I've employed the <code>cpu_usage</code> condition. When CPU usage for a Mongrel process is over 50% for 5 consecutive intervals, it will be restarted.</p>
+
+<pre><code class="ruby">    w.lifecycle do |on|
+      on.condition(:flapping) do |c|
+        c.to_state = [:start, :restart]
+        c.times = 5
+        c.within = 5.minute
+        c.transition = :unmonitored
+        c.retry_in = 10.minutes
+        c.retry_times = 5
+        c.retry_within = 2.hours
+      end
+    end</code></pre>
+    
+<p>Conditions inside a <code>lifecycle</code> section are active as long as the process is being monitored (they live across state changes). 
+  
+<p>The <code>:flapping</code> condition guards against the edge case wherein god rapidly starts or restarts your application. Things like server configuration changes or the unavailability of external services could make it impossible for my process to start. In that case, god will try to start my process over and over to no avail. The <code>:flapping</code> condition provides two levels of giving up on flapping processes. If I were to translate the options of the code above, it would be something like: If this watch is started or restarted five times withing 5 minutes, then unmonitor it...then after ten minutes, monitor it again to see if it was just a temporary problem; if the process is seen to be flapping five times within two hours, then give up completely.</p>
+
+<p>That's it!</p>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Changing UID/GID for processes</h1>
+
+<p>It is possible to have god run your start/stop/restart commands as a specific user/group. This can be done by setting the <code>uid</code> and/or <code>gid</code> attributes of a watch.</p>
+
+<pre><code class="ruby">  God.watch do |w|
+    ...
+    
+    w.uid = 'tom'
+    w.gid = 'devs'
+    
+    ...
+  end</code></pre>
+    
+</p>This only works for commands specified as a string. Lambda commands are unaffected.</p>
+
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Lambda commands</h1>
+
+<p>In addition to specifying start/stop/restart commands as strings (to be executed via the shell), you can specify a lambda that will be called.</p>
+
+<pre><code class="ruby">  God.watch do |w|
+    ...
+    
+    w.start = lambda { ENV['APACHE'] ? `apachectl -k graceful` : `lighttpd restart` }
+    
+    ...
+  end</code></pre>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Starting and Controlling God</h1>
+
+<p>To start the god monitoring process as a daemon simply run the <code>god</code> executable passing in the path to the config file (you need to sudo if you're using events on Linux or want to use the setuid/setgid functionality):</p>
+
+<pre>$ sudo god -c /path/to/config.god</pre>
+
+<p>While you're writing your config file, it can be helpful to run god in the foreground so you can see the log messages. You can do that with:</p>
+
+<pre>$ sudo god -c /path/to/config.god -D</pre>
+
+<p>You can start/restart/stop/monitor/unmonitor your Watches with the same utility like so:</p>
+
+<pre>$ sudo god stop gravatar2-mongrel-8200</pre>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Grouping Watches</h1>
+
+<p>Watches can be assigned to groups. These groups can then be controlled together from the command line.</p>
+
+<pre><code class="ruby">  God.watch do |w|
+    ...
+    
+    w.group = 'mongrels'
+    
+    ...
+  end</code></pre>
+    
+<p>The above configuration now allows you to control the watch (and any others that are in the group) with a single command:</p>
+
+<pre>$ sudo god stop mongrels</pre>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Advanced Configuration with Transitions and Events</h1>
+
+<p>So far you've been introduced to a simple poll-based config file and seen how to run it. Poll-based monitoring works great for simple things, but falls short for highly critical tasks. God has native support for kqueue/netlink events on BSD/Darwin/Linux systems. For instance, instead of using the <code>process_running</code> condition to poll for the status of your process, you can use the <code>process_exits</code> condition that will be notified <b>immediately</b> upon the exit of your process. This means less load on your system and shorter downtime after a crash.</p>
+
+<p>While the configuration syntax you saw in the previous example is very simple, it lacks the power that we need to deal with event based monitoring. In fact, the <code>start_if</code> and <code>restart_if</code> methods are really just calling out to a lower-level API. If we use the low-level API directly, we can harness the full power of god's event based lifecycle system. Let's look at another example config file.</p>
+
+<pre><code class="ruby">RAILS_ROOT = "/Users/tom/dev/gravatar2"
+
+God.watch do |w|
+  w.name = "local-3000"
+  w.interval = 5.seconds # default
+  w.start = "mongrel_rails start -c #{RAILS_ROOT} -P #{RAILS_ROOT}/log/mongrel.pid -p 3000 -d"
+  w.stop = "mongrel_rails stop -P #{RAILS_ROOT}/log/mongrel.pid"
+  w.restart = "mongrel_rails restart -P #{RAILS_ROOT}/log/mongrel.pid"
+  w.pid_file = File.join(RAILS_ROOT, "log/mongrel.pid")
+  
+  # clean pid files before start if necessary
+  w.behavior(:clean_pid_file)
+  
+  # determine the state on startup
+  w.transition(:init, { true => :up, false => :start }) do |on|
+    on.condition(:process_running) do |c|
+      c.running = true
+    end
+  end
+  
+  # determine when process has finished starting
+  w.transition([:start, :restart], :up) do |on|
+    on.condition(:process_running) do |c|
+      c.running = true
+    end
+    
+    # failsafe
+    on.condition(:tries) do |c|
+      c.times = 5
+      c.transition = :start
+    end
+  end
+
+  # start if process is not running
+  w.transition(:up, :start) do |on|
+    on.condition(:process_exits)
+  end
+  
+  # restart if memory or cpu is too high
+  w.transition(:up, :restart) do |on|
+    on.condition(:memory_usage) do |c|
+      c.interval = 20
+      c.above = 50.megabytes
+      c.times = [3, 5]
+    end
+    
+    on.condition(:cpu_usage) do |c|
+      c.interval = 10
+      c.above = 10.percent
+      c.times = [3, 5]
+    end
+  end
+  
+  # lifecycle
+  w.lifecycle do |on|
+    on.condition(:flapping) do |c|
+      c.to_state = [:start, :restart]
+      c.times = 5
+      c.within = 5.minute
+      c.transition = :unmonitored
+      c.retry_in = 10.minutes
+      c.retry_times = 5
+      c.retry_within = 2.hours
+    end
+  end
+end
+</code></pre>
+
+<p>A bit longer, I know, but very straighforward once you understand how the <code>transition</code> calls work. The <code>name</code>, <code>interval</code>, <code>start</code>, <code>stop</code>, and <code>pid_file</code> attributes should be familiar. We also specify the <code>clean_pid_file</code> behavior.</p>
+
+<p>Before jumping into the code, it's important to understand the different states that a Watch can have, and how that state changes over time. At any given time, a Watch will be in one of the <code>init</code>, <code>up</code>, <code>start</code>, or <code>restart</code> states. As different conditions are satisfied, the Watch will progress from state to state, enabling and disabling conditions along the way.</p>
+  
+<p>When god first starts, each Watch is placed in the <code>init</code> state.</p>
+
+<p>You'll use the <code>transition</code> method to tell god how to transition between states. It takes two arguments. The first argument may be either a symbol or an array of symbols representing the state or states during which the specified conditions should be enabled. The second argument may be either a symbol or a hash. If it is a symbol, then that is the state that will be transitioned to if any of the conditions return <code>true</code>. If it is a hash, then that hash must have both <code>true</code> and <code>false</code> keys, each of which point to a symbol that represents the state to transition to given the corresponding return from the single condition that must be specified.</p>
+
+<pre><code class="ruby">  # determine the state on startup
+  w.transition(:init, { true => :up, false => :start }) do |on|
+    on.condition(:process_running) do |c|
+      c.running = true
+    end
+  end</code></pre>
+
+<p>The first transition block tells god what to do when the Watch is in the <code>init</code> state (first argument). This is where I tell god how to determine if my task is already running. Since I'm monitoring a process, I can use the <code>process_running</code> condition to determine whether the process is running. If the process is running, it will return true, otherwise it will return false. Since I sent a hash as the second argument to <code>transition</code>, the return from <code>process_running</code> will determine which of the two states will be transitioned to. If the process is running, the return is true and god will put the Watch into the <code>up</code> state. If the process is not running, the return is false and god will put the Watch into the <code>start</code> state.</p>
+
+<pre><code class="ruby">  # determine when process has finished starting
+  w.transition([:start, :restart], :up) do |on|
+    on.condition(:process_running) do |c|
+      c.running = true
+    end
+    
+    ...
+  end</code></pre>
+
+<p>If god has determined that my process isn't running, the Watch will be put into the <code>start</code> state. Upon entering this state, the <code>start</code> command that I specified on the Watch will be called. In addition, the above transition specifies a condition that should be enabled when in either the <code>start</code> or <code>restart</code> states. The condition is another <code>process_running</code>, however this time I'm only interested in moving to another state once it returns <code>true</code>. A <code>true</code> return from this condition means that the process is running and it's ok to transition to the <code>up</code> state (second argument to <code>transition</code>).</p>
+
+<pre><code class="ruby">  # determine when process has finished starting
+  w.transition([:start, :restart], :up) do |on|
+    ...
+    
+    # failsafe
+    on.condition(:tries) do |c|
+      c.times = 5
+      c.transition = :start
+    end
+  end</code></pre>
+
+<p>The other half of this transition uses the <code>tries</code> condition to ensure that god doesn't get stuck in this state. It's possible that the process could go down while the transition is being made, in which case god would end up polling forever to see if the process is up. Here I've specified that if this condition is called five times, god should override the normal transition destination and move to the <code>start</code> state instead. If you specify a <code>transition</code> attribute on any condition, that state will be transferred to instead of the normal transfer destination.</p>
+
+<pre><code class="ruby">  # start if process is not running
+  w.transition(:up, :start) do |on|
+    on.condition(:process_exits)
+  end</code></pre>
+
+<p>This is where the event based system comes into play. Once in the <code>up</code> state, I want to be notified when my process exits. The <code>process_exits</code> condition registers a callback that will trigger a transition change when it is fired off. Event conditions (like this one) cannot be used in transitions that have a hash for the second argument (as they do not return true or false).</p>
+
+<pre><code class="ruby">  # restart if memory or cpu is too high
+  w.transition(:up, :restart) do |on|
+    on.condition(:memory_usage) do |c|
+      c.interval = 20
+      c.above = 50.megabytes
+      c.times = [3, 5]
+    end
+    
+    on.condition(:cpu_usage) do |c|
+      c.interval = 10
+      c.above = 10.percent
+      c.times = [3, 5]
+    end
+  end</code></pre>
+
+<p>Notice that I can have multiple transitions with the same start state. In this case, I want to have the <code>memory_usage</code> and <code>cpu_usage</code> poll conditions going at the same time that I listen for the process exit event. In the case of runaway CPU or memory usage, however, I want to transition to the <code>restart</code> state. When a Watch enters the <code>restart</code> state it will either call the <code>restart</code> command that you specified, or if none has been set, call the <code>stop</code> and then <code>start</code> commands.</p>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Watching Non-Daemon Processes</h1>
+
+<p>Need to watch a script that doesn't have built in daemonization? No problem! God will daemonize and keep track of your process for you. If you don't specify a <code>pid_file</code> attribute for a watch, it will be auto-daemonized and a PID file will be stored for it in <code>/var/run/god</code>. If you'd rather have the PID file stored in a different location, you can set it at the top of your config:</p>
+
+<pre><code class="ruby">God.pid_file_directory = '/home/tom/pids'
+
+God.watch do |w|
+  # watch with no pid_file attribute set
+end</code></pre>
+
+<p>The directory you specify must be writable by god.</p>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Loading Other Config Files</h1>
+
+<p>You should feel free to separate your god configs into separate files for easier organization. You can load in other configs using Ruby's normal <code>load</code> method, or use the convenience method <code>God.load</code> which allows for glob-style paths:</p>
+
+<pre><code class="ruby"># load in all god configs
+God.load "/usr/local/conf/*.god"</code></pre>
+
+<p>God won't start its monitoring operations until all configurations have been loaded.</p>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Dynamically Loading Config Files Into an Already Running God</h1>
+
+<p>God allows you to load or reload configurations into an already running instance. There are a few things to consider when doing this:</p>
+
+<ul>
+  <li>Existng Watches with the same <code>name</code> as the incoming Watches will be overidden by the new config.</li>
+  <li>All paths must be either absolute or relative to the path from which god was started.</li>
+</ul>
+
+<p>To load a config into a running god, issue the following command:</p>
+
+<pre>$ sudo god load path/to/config.god</pre>
+
+<p>Config files that are loaded dynamically can contain anything that a normal config file contains, however, global options such as <code>God.pid_file_directory</code> blocks will be ignored (and produce a warning in the logs).</p>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Getting Logs for a Single Watch</h1>
+
+<p>Sifting through the god logs for statements specific to a single Watch can be frustrating when you have many of them. You can get the realtime logs for a single Watch via the command line:</p>
+
+<pre>$ sudo god log local-3000</pre>
+
+<p>This will display log output for the 'local-3000' Watch and update every second with new log messages.</p>
+
+<p>You can also supply a shorthand to the log command that will match one of your watches. If it happens to match several, the shortest match will be used:</p>
+
+<pre>$ sudo god log l3</pre>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Notifications</h1>
+
+<p>God has an extensible notification framework built in that makes it easy to have notifications sent when conditions are triggered. There are three steps to enabling notifications.</p>
+
+<p>Step 1: Set the options for the notification subsystem(s) that you'll be using. Let's look at how to setup email notifications.</p>
+
+<pre><code class="ruby">God::Contacts::Email.message_settings = {
+  :from => 'god@example.com'
+}
+
+God::Contacts::Email.server_settings = {
+  :address => "smtp.example.com",
+  :port => 25,
+  :domain => "example.com",
+  :authentication => :plain,
+  :user_name => "john",
+  :password => "s3kr3ts"
+}</code></pre>
+
+<p>Step 2: Configure some contacts.</p>
+
+<pre><code class="ruby">God.contact(:email) do |c|
+  c.name = 'tom'
+  c.email = 'tom@example.com'
+end
+
+God.contact(:email) do |c|
+  c.name = 'vanpelt'
+  c.email = 'vanpelt@example.com'
+  c.group = 'developers'
+end
+
+God.contact(:email) do |c|
+  c.name = 'kevin'
+  c.email = 'kevin@example.com'
+  c.group = 'developers'
+end</code></pre>
+
+<p>Step 3: Attach to a condition:</p>
+
+<pre><code class="ruby">  w.transition(:up, :start) do |on|
+    on.condition(:process_exits) do |c|
+      c.notify = 'tom'
+    end
+  end</code></pre>
+  
+<p>There are two ways to specify that a notification should be sent. The first, easier way is shown above. Every condition can take an optional <code>notify</code> attribute that specifies which contacts should be notified when the condition is triggered. The value can be a contact name or contact group *or* an array of contact names and/or contact groups.</p>
+  
+<pre><code class="ruby">  w.transition(:up, :start) do |on|
+    on.condition(:process_exits) do |c|
+      c.notify = {:contacts => ['tom', 'developers'], :priority => 1, :category => 'product'}
+    end
+  end</code></pre>
+  
+</p>The second way allows you to specify the <code>priority</code> and <code>category</code> in addition to the contacts. The extra attributes can be arbitrary integers or strings and will be passed as-is to the notification subsystem.</p>
+
+<p>The above notification will arrive as an email similar to the following.</p>
+
+<pre><code>From: god &lt;god@example.com&gt;
+To: tom &lt;tom@example.com&gt;
+Subject: [god] mongrel-8600 [trigger] process exited (ProcessExits)
+
+Message: mongrel-8600 [trigger] process exited (ProcessExits)
+Host: candymountain.example.com
+Priority: 1
+Category: product</code></pre>
+
+<!-- ------------------------------------------------------------------------- -->
+
+<h1>Extend God with your own Conditions</h1>
+
+<p>God was designed from the start to allow you to easily write your own custom conditions, making it simple to add tests that are application specific.</p>
+
+</div>
+<div id="footer">
+	<p>Brought to you by <a href="http://tom.preston-werner.com/">Tom Preston-Werner</a></p>
+</div>
+
+<script src="http://www.google-analytics.com/urchin.js" type="text/javascript">
+</script>
+<script type="text/javascript">
+_uacct = "UA-2196727-1";
+urchinTracker();
+</script>
+
+</body>
+</html>
\ No newline at end of file
diff -pruN 0.7.13-3/site/install.html 0.7.18-1/site/install.html
--- 0.7.13-3/site/install.html	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/site/install.html	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,2 @@
+ln -s /usr/src/linux-headers-2.6.15-28/include/linux/connector.h /usr/include/linux/connector.h
+ln -s /usr/src/linux-headers-2.6.15-28/include/linux/cn_proc.h /usr/include/linux/cn_proc.h
\ No newline at end of file
diff -pruN 0.7.13-3/site/javascripts/code_highlighter.js 0.7.18-1/site/javascripts/code_highlighter.js
--- 0.7.13-3/site/javascripts/code_highlighter.js	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/site/javascripts/code_highlighter.js	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,188 @@
+/* Unobtrustive Code Highlighter By Dan Webb 11/2005
+   Version: 0.4
+	
+	Usage:
+		Add a script tag for this script and any stylesets you need to use
+		to the page in question, add correct class names to CODE elements, 
+		define CSS styles for elements. That's it! 
+	
+	Known to work on:
+		IE 5.5+ PC
+		Firefox/Mozilla PC/Mac
+		Opera 7.23 + PC
+		Safari 2
+		
+	Known to degrade gracefully on:
+		IE5.0 PC
+	
+	Note: IE5.0 fails due to the use of lookahead in some stylesets.  To avoid script errors
+	in older browsers use expressions that use lookahead in string format when defining stylesets.
+	
+	This script is inspired by star-light by entirely cunning Dean Edwards
+	http://dean.edwards.name/star-light/.  
+*/
+
+// replace callback support for safari.
+if ("a".replace(/a/, function() {return "b"}) != "b") (function(){
+  var default_replace = String.prototype.replace;
+  String.prototype.replace = function(search,replace){
+	// replace is not function
+	if(typeof replace != "function"){
+		return default_replace.apply(this,arguments)
+	}
+	var str = "" + this;
+	var callback = replace;
+	// search string is not RegExp
+	if(!(search instanceof RegExp)){
+		var idx = str.indexOf(search);
+		return (
+			idx == -1 ? str :
+			default_replace.apply(str,[search,callback(search, idx, str)])
+		)
+	}
+	var reg = search;
+	var result = [];
+	var lastidx = reg.lastIndex;
+	var re;
+	while((re = reg.exec(str)) != null){
+		var idx  = re.index;
+		var args = re.concat(idx, str);
+		result.push(
+			str.slice(lastidx,idx),
+			callback.apply(null,args).toString()
+		);
+		if(!reg.global){
+			lastidx += RegExp.lastMatch.length;
+			break
+		}else{
+			lastidx = reg.lastIndex;
+		}
+	}
+	result.push(str.slice(lastidx));
+	return result.join("")
+  }
+})();
+
+var CodeHighlighter = { styleSets : new Array };
+
+CodeHighlighter.addStyle = function(name, rules) {
+	// using push test to disallow older browsers from adding styleSets
+	if ([].push) this.styleSets.push({
+		name : name, 
+		rules : rules,
+		ignoreCase : arguments[2] || false
+	})
+	
+	function setEvent() {
+		// set highlighter to run on load (use LowPro if present)
+		if (typeof Event != 'undefined' && typeof Event.onReady == 'function') 
+		  return Event.onReady(CodeHighlighter.init.bind(CodeHighlighter));
+		
+		var old = window.onload;
+		
+		if (typeof window.onload != 'function') {
+			window.onload = function() { CodeHighlighter.init() };
+		} else {
+			window.onload = function() {
+				old();
+				CodeHighlighter.init();
+			}
+		}
+	}
+	
+	// only set the event when the first style is added
+	if (this.styleSets.length==1) setEvent();
+}
+
+CodeHighlighter.init = function() {
+	if (!document.getElementsByTagName) return; 
+	if ("a".replace(/a/, function() {return "b"}) != "b") return; // throw out Safari versions that don't support replace function
+	// throw out older browsers
+	
+	var codeEls = document.getElementsByTagName("CODE");
+	// collect array of all pre elements
+	codeEls.filter = function(f) {
+		var a =  new Array;
+		for (var i = 0; i < this.length; i++) if (f(this[i])) a[a.length] = this[i];
+		return a;
+	} 
+	
+	var rules = new Array;
+	rules.toString = function() {
+		// joins regexes into one big parallel regex
+		var exps = new Array;
+		for (var i = 0; i < this.length; i++) exps.push(this[i].exp);
+		return exps.join("|");
+	}
+	
+	function addRule(className, rule) {
+		// add a replace rule
+		var exp = (typeof rule.exp != "string")?String(rule.exp).substr(1, String(rule.exp).length-2):rule.exp;
+		// converts regex rules to strings and chops of the slashes
+		rules.push({
+			className : className,
+			exp : "(" + exp + ")",
+			length : (exp.match(/(^|[^\\])\([^?]/g) || "").length + 1, // number of subexps in rule
+			replacement : rule.replacement || null 
+		});
+	}
+	
+	function parse(text, ignoreCase) {
+		// main text parsing and replacement
+		return text.replace(new RegExp(rules, (ignoreCase)?"gi":"g"), function() {
+			var i = 0, j = 1, rule;
+			while (rule = rules[i++]) {
+				if (arguments[j]) {
+					// if no custom replacement defined do the simple replacement
+					if (!rule.replacement) return "<span class=\"" + rule.className + "\">" + arguments[0] + "</span>";
+					else {
+						// replace $0 with the className then do normal replaces
+						var str = rule.replacement.replace("$0", rule.className);
+						for (var k = 1; k <= rule.length - 1; k++) str = str.replace("$" + k, arguments[j + k]);
+						return str;
+					}
+				} else j+= rule.length;
+			}
+		});
+	}
+	
+	function highlightCode(styleSet) {
+		// clear rules array
+		var parsed, clsRx = new RegExp("(\\s|^)" + styleSet.name + "(\\s|$)");
+		rules.length = 0;
+		
+		// get stylable elements by filtering out all code elements without the correct className	
+		var stylableEls = codeEls.filter(function(item) { return clsRx.test(item.className) });
+		
+		// add style rules to parser
+		for (var className in styleSet.rules) addRule(className, styleSet.rules[className]);
+		
+			
+		// replace for all elements
+		for (var i = 0; i < stylableEls.length; i++) {
+			// EVIL hack to fix IE whitespace badness if it's inside a <pre>
+			if (/MSIE/.test(navigator.appVersion) && stylableEls[i].parentNode.nodeName == 'PRE') {
+				stylableEls[i] = stylableEls[i].parentNode;
+				
+				parsed = stylableEls[i].innerHTML.replace(/(<code[^>]*>)([^<]*)<\/code>/i, function() {
+					return arguments[1] + parse(arguments[2], styleSet.ignoreCase) + "</code>"
+				});
+				parsed = parsed.replace(/\n( *)/g, function() { 
+					var spaces = "";
+					for (var i = 0; i < arguments[1].length; i++) spaces+= "&nbsp;";
+					return "\n" + spaces;  
+				});
+				parsed = parsed.replace(/\t/g, "&nbsp;&nbsp;&nbsp;&nbsp;");
+				parsed = parsed.replace(/\n(<\/\w+>)?/g, "<br />$1").replace(/<br \/>[\n\r\s]*<br \/>/g, "<p><br></p>");
+				
+			} else parsed = parse(stylableEls[i].innerHTML, styleSet.ignoreCase);
+			
+			stylableEls[i].innerHTML = parsed;
+		}
+	}
+	
+	// run highlighter on all stylesets
+	for (var i=0; i < this.styleSets.length; i++) {
+		highlightCode(this.styleSets[i]);  
+	}
+}
\ No newline at end of file
diff -pruN 0.7.13-3/site/javascripts/ruby.js 0.7.18-1/site/javascripts/ruby.js
--- 0.7.13-3/site/javascripts/ruby.js	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/site/javascripts/ruby.js	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,18 @@
+CodeHighlighter.addStyle("ruby",{
+	comment : {
+		exp  : /#[^\n]+/
+	},
+	brackets : {
+		exp  : /\(|\)/
+	},
+	string : {
+		exp  : /'[^']*'|"[^"]*"/
+	},
+	keywords : {
+		exp  : /\b(do|end|self|class|def|if|module|yield|then|else|for|until|unless|while|elsif|case|when|break|retry|redo|rescue|require|raise)\b/
+	},
+	/* Added by Shelly Fisher (shelly@agileevolved.com) */
+	symbol : {
+	  exp : /([^:])(:[A-Za-z0-9_!?]+)/
+	}
+});
\ No newline at end of file
diff -pruN 0.7.13-3/site/stylesheets/layout.css 0.7.18-1/site/stylesheets/layout.css
--- 0.7.13-3/site/stylesheets/layout.css	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/site/stylesheets/layout.css	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,174 @@
+* {
+	margin: 0;
+	font-size: 100%;
+}
+
+body {
+	font: normal .8em/1.5em "Trebuchet MS", Verdana, Arial, Helvetica, sans-serif;
+	color: #484848;
+	background: #E6EAE9 url(../images/bg_grey.gif);
+}
+
+a {
+	color: #c75f3e;
+	text-decoration: none;
+}
+
+a:hover,
+a:active {
+	text-decoration: underline;
+}
+
+#mothership {
+	width: 307px;
+	height: 117px;
+	margin: 0 auto;
+	background: url(../images/god_logo.png);
+}
+
+#content {
+	width: 700px;
+	margin: 3px auto;
+	background: white;
+	border: 1px solid #444;
+	padding: 0 24px;
+	background: #f8f8ff;
+	overflow: hidden;
+}
+
+.banner {
+	margin-top: 24px;
+	border: 1px solid #ddd;
+	width: 698px;
+	height: 150px;
+	background: url(../images/banner.jpg);
+}
+
+#menu {
+	margin-top: 5px;
+}
+
+#menu div.dots {
+	background: url(../images/red_dot.gif) repeat;
+	height: 5px;
+	width: 700px;
+	font-size: 0;
+}
+
+#menu ul {
+	font-family: "Trebuchet MS", Verdana, Arial, Helvetica, sans-serif;
+	font-weight: bold;
+	text-transform: uppercase;
+	color: #4D4D4D;
+	font-size: 12px;
+	padding: 0;
+	margin: 0;
+	margin-top: 0 !important;
+	margin-top: -2px;
+}
+
+#menu li {
+	display: inline;
+	margin: 0 30px 0 0;
+}
+
+#menu a:link,
+#menu a:visited {
+	color: #4D4D4D;
+	text-decoration: none;
+}
+
+#menu a:hover,
+#menu a:active {
+	color: black;
+	text-decoration: none;
+}
+
+#page_home #menu li.menu_home a {
+	color: #A70000;
+}
+
+.columnleft {
+	float: left;
+	width: 345px;
+	margin-bottom: 20px;
+}
+
+.columnleft p {
+		text-align: justify;
+}
+
+.columnright {
+	float: right;
+	width: 325px;
+	margin-bottom: 20px;
+}
+
+h1 {
+	font: bold 1.5em "Trebuchet MS", Verdana, Arial, Helvetica, sans-serif;
+	color: #f36e21;
+	text-transform: uppercase;
+	margin: 1.5em 0 .5em 0;
+	clear: both;
+}
+
+p {
+	margin-bottom: 1em;
+}
+
+ul {
+  margin-bottom: 1em;
+}
+
+ul.features {
+	padding: 0;
+	margin-left: 1.5em !important;
+	margin-left: 1.3em;
+}
+
+ul.features li {
+	list-style-position: outside;
+	list-style-type: circle;
+	list-style-image: url(../images/bullet.jpg);
+	line-height: 1.4em;
+}
+
+#footer {
+	text-align: center;
+	color: white;
+	margin-bottom: 50px;
+}
+
+
+
+pre {
+  line-height: 1.3;
+  border: 1px solid #ccc;
+  padding: 1em;
+  background-color: #efefef;
+  margin: 1em 0;
+}
+
+code {
+  font-size: 1.2em;
+}
+
+.ruby .keywords {
+  color: blue;
+}
+
+.ruby .comment {
+	color : green;
+}
+
+.ruby .string {
+	color : teal;
+}
+
+.ruby .keywords {
+	color : navy;
+}
+
+.ruby .brackets {
+	color : navy;
+}
\ No newline at end of file
Binary files 0.7.13-3/test/configs/child_events/._child_events.god and 0.7.18-1/test/configs/child_events/._child_events.god differ
Binary files 0.7.13-3/test/configs/child_events/._simple_server.rb and 0.7.18-1/test/configs/child_events/._simple_server.rb differ
diff -pruN 0.7.13-3/test/configs/child_polls/child_polls.god 0.7.18-1/test/configs/child_polls/child_polls.god
--- 0.7.13-3/test/configs/child_polls/child_polls.god	2009-02-20 01:04:57.000000000 +0000
+++ 0.7.18-1/test/configs/child_polls/child_polls.god	2010-02-21 14:01:37.000000000 +0000
@@ -1,6 +1,6 @@
 God.watch do |w|
   w.name = 'child-polls'
-  w.start = File.join(GOD_ROOT, *%w[test configs child_polls simple_server.rb])
+  w.start = File.join(GOD_ROOT, *%w[test configs child_polls simple_server.rb]) + ' ' + ENV['RAILS_ENV']
   w.interval = 5
   w.grace = 2
   
@@ -34,4 +34,4 @@ God.watch do |w|
       c.retry_within = 5.minutes
     end
   end
-end
\ No newline at end of file
+end
diff -pruN 0.7.13-3/test/configs/lifecycle/lifecycle.god 0.7.18-1/test/configs/lifecycle/lifecycle.god
--- 0.7.13-3/test/configs/lifecycle/lifecycle.god	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/test/configs/lifecycle/lifecycle.god	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,25 @@
+God::Contacts::Twitter.settings = {
+  # this is for my 'mojombo2' twitter test account
+  # feel free to use it for testing your conditions
+  :username => 'mojombo@gmail.com',
+  :password  => 'gok9we3ot1av2e'
+}
+
+God.contact(:twitter) do |c|
+  c.name      = 'tom2'
+  c.group     = 'developers'
+end
+
+God.watch do |w|
+  w.name = "lifecycle"
+  w.interval = 5.seconds
+  w.start = "/dev/null"
+  
+  # lifecycle
+  w.lifecycle do |on|
+    on.condition(:always) do |c|
+      c.what = true
+      c.notify = "tom2"
+    end
+  end
+end
\ No newline at end of file
Binary files 0.7.13-3/test/configs/task/._task.god and 0.7.18-1/test/configs/task/._task.god differ
diff -pruN 0.7.13-3/test/helper.rb 0.7.18-1/test/helper.rb
--- 0.7.13-3/test/helper.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/helper.rb	2010-02-21 14:01:37.000000000 +0000
@@ -90,19 +90,7 @@ ensure
   $VERBOSE = old_verbose
 end
 
-def no_stdout
-  old_stdout = $stdout.dup
-  $stdout.reopen(File.open((PLATFORM =~ /mswin/ ? "NUL" : "/dev/null"), 'w'))
-  yield
-  $stdout.reopen(old_stdout)
-end
-
-def no_stderr
-  old_stderr = $stderr.dup
-  $stderr.reopen(File.open((PLATFORM =~ /mswin/ ? "NUL" : "/dev/null"), 'w'))
-  yield
-  $stderr.reopen(old_stderr)
-end
+LOG.instance_variable_set(:@io, StringIO.new('/dev/null'))
 
 module Kernel
   def abort(text)
diff -pruN 0.7.13-3/test/test_behavior.rb 0.7.18-1/test/test_behavior.rb
--- 0.7.13-3/test/test_behavior.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_behavior.rb	2010-02-21 14:01:37.000000000 +0000
@@ -13,9 +13,6 @@ class TestBehavior < Test::Unit::TestCas
   
   def test_complain
     Syslog.expects(:err).with('foo')
-    # Kernel.expects(:puts).with('foo')
-    no_stdout do
-      assert !Behavior.allocate.bypass.complain('foo')
-    end
+    assert !Behavior.allocate.bypass.complain('foo')
   end
 end
\ No newline at end of file
diff -pruN 0.7.13-3/test/test_conditions_disk_usage.rb 0.7.18-1/test/test_conditions_disk_usage.rb
--- 0.7.13-3/test/test_conditions_disk_usage.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_conditions_disk_usage.rb	2010-02-21 14:01:37.000000000 +0000
@@ -7,20 +7,14 @@ class TestConditionsDiskUsage < Test::Un
     c = Conditions::DiskUsage.new
     c.mount_point = '/'
     c.watch = stub(:name => 'foo')
-    
-    no_stdout do
-      assert_equal false, c.valid?
-    end
+    assert_equal false, c.valid?
   end
   
   def test_valid_should_return_false_if_no_mount_point_given
     c = Conditions::DiskUsage.new
     c.above = 90
     c.watch = stub(:name => 'foo')
-    
-    no_stdout do
-      assert_equal false, c.valid?
-    end
+    assert_equal false, c.valid?
   end
   
   def test_valid_should_return_true_if_required_options_all_set
diff -pruN 0.7.13-3/test/test_conditions_http_response_code.rb 0.7.18-1/test/test_conditions_http_response_code.rb
--- 0.7.13-3/test/test_conditions_http_response_code.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_conditions_http_response_code.rb	2010-02-21 14:01:37.000000000 +0000
@@ -26,14 +26,14 @@ class TestHttpResponseCode < Test::Unit:
     c = valid_condition do |cc|
       cc.code_is_not = 500
     end
-    no_stdout { assert !c.valid? }
+    assert !c.valid?
   end
   
   def test_valid_should_return_false_if_no_host_set
     c = valid_condition do |cc|
       cc.host = nil
     end
-    no_stdout { assert !c.valid? }
+    assert !c.valid?
   end
   
   # test
diff -pruN 0.7.13-3/test/test_conditions_process_running.rb 0.7.18-1/test/test_conditions_process_running.rb
--- 0.7.13-3/test/test_conditions_process_running.rb	2008-11-14 01:35:31.000000000 +0000
+++ 0.7.18-1/test/test_conditions_process_running.rb	2010-02-21 14:01:37.000000000 +0000
@@ -5,12 +5,8 @@ class TestConditionsProcessRunning < Tes
     [true, false].each do |r|
       c = Conditions::ProcessRunning.new
       c.running = r
-    
       c.stubs(:watch).returns(stub(:pid => 99999999, :name => 'foo'))
-    
-      # no_stdout do
-        assert_equal !r, c.test
-      # end
+      assert_equal !r, c.test
     end
   end
   
diff -pruN 0.7.13-3/test/test_conditions_tries.rb 0.7.18-1/test/test_conditions_tries.rb
--- 0.7.13-3/test/test_conditions_tries.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_conditions_tries.rb	2010-02-21 14:01:37.000000000 +0000
@@ -6,7 +6,7 @@ class TestConditionsTries < Test::Unit::
   def test_valid_should_return_false_if_times_not_set
     c = Conditions::Tries.new
     c.watch = stub(:name => 'foo')
-    no_stdout { assert !c.valid? }
+    assert !c.valid?
   end
 end
 
diff -pruN 0.7.13-3/test/test_god.rb 0.7.18-1/test/test_god.rb
--- 0.7.13-3/test/test_god.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_god.rb	2010-02-21 14:01:37.000000000 +0000
@@ -164,9 +164,7 @@ class TestGod < Test::Unit::TestCase
     God.running = true
     
     assert_nothing_raised do
-      no_stdout do
-        God.watch { |w| w.name = 'foo'; w.start = 'bar' }
-      end
+      God.watch { |w| w.name = 'foo'; w.start = 'bar' }
     end
   end
   
@@ -177,26 +175,20 @@ class TestGod < Test::Unit::TestCase
     w = God.watches['bar']
     w.state = :up
     w.expects(:unmonitor)
-    no_stdout do
-      God.unwatch(w)
-    end
+    God.unwatch(w)
   end
   
   def test_unwatch_should_unregister_watch
     God.watch { |w| w.name = 'bar'; w.start = 'bar' }
     w = God.watches['bar']
     w.expects(:unregister!)
-    no_stdout do
-      God.unwatch(w)
-    end
+    God.unwatch(w)
   end
   
   def test_unwatch_should_remove_same_name_watches
     God.watch { |w| w.name = 'bar'; w.start = 'bar' }
     w = God.watches['bar']
-    no_stdout do
-      God.unwatch(w)
-    end
+    God.unwatch(w)
     assert_equal 0, God.watches.size
   end
   
@@ -207,9 +199,7 @@ class TestGod < Test::Unit::TestCase
       w.group = 'test'
     end
     w = God.watches['bar']
-    no_stdout do
-      God.unwatch(w)
-    end
+    God.unwatch(w)
     assert !God.groups[w.group].include?(w)
   end
   
@@ -239,28 +229,22 @@ class TestGod < Test::Unit::TestCase
   end
   
   def test_contact_should_abort_on_no_name
-    no_stdout do
-      assert_abort do
-        God.contact(:fake_contact) { |c| }
-      end
+    assert_abort do
+      God.contact(:fake_contact) { |c| }
     end
   end
   
   def test_contact_should_abort_on_duplicate_contact_name
     God.contact(:fake_contact) { |c| c.name = 'tom' }
-    no_stdout do
-      assert_nothing_raised do
-        God.contact(:fake_contact) { |c| c.name = 'tom' }
-      end
+    assert_nothing_raised do
+      God.contact(:fake_contact) { |c| c.name = 'tom' }
     end
   end
   
   def test_contact_should_abort_on_contact_with_same_name_as_group
     God.contact(:fake_contact) { |c| c.name = 'tom'; c.group = 'devs' }
-    no_stdout do
-      assert_nothing_raised do
-        God.contact(:fake_contact) { |c| c.name = 'devs' }
-      end
+    assert_nothing_raised do
+      God.contact(:fake_contact) { |c| c.name = 'devs' }
     end
   end
   
@@ -422,9 +406,7 @@ class TestGod < Test::Unit::TestCase
       end
     EOF
     
-    no_stdout do
-      God.running_load(code, '/foo/bar.god')
-    end
+    God.running_load(code, '/foo/bar.god')
     
     assert_equal 1, God.watches.size
   end
@@ -438,9 +420,7 @@ class TestGod < Test::Unit::TestCase
     EOF
     
     Watch.any_instance.expects(:monitor)
-    no_stdout do
-      God.running_load(code, '/foo/bar.god')
-    end
+    God.running_load(code, '/foo/bar.god')
   end
   
   def test_running_load_should_not_monitor_new_watches_with_autostart_false
@@ -453,9 +433,7 @@ class TestGod < Test::Unit::TestCase
     EOF
     
     Watch.any_instance.expects(:monitor).never
-    no_stdout do
-      God.running_load(code, '/foo/bar.god')
-    end
+    God.running_load(code, '/foo/bar.god')
   end
   
   def test_running_load_should_return_array_of_affected_watches
@@ -467,9 +445,7 @@ class TestGod < Test::Unit::TestCase
     EOF
     
     w = nil
-    no_stdout do
-      w, e = *God.running_load(code, '/foo/bar.god')
-    end
+    w, e = *God.running_load(code, '/foo/bar.god')
     assert_equal 1, w.size
     assert_equal 'foo', w.first
   end
@@ -482,9 +458,7 @@ class TestGod < Test::Unit::TestCase
       end
     EOF
     
-    no_stdout do
-      God.running_load(code, '/foo/bar.god')
-    end
+    God.running_load(code, '/foo/bar.god')
     assert_equal 0, God.pending_watches.size
   end
   
@@ -527,9 +501,7 @@ class TestGod < Test::Unit::TestCase
   
   def test_start_should_get_and_join_timer
     God.watch { |w| w.name = 'foo'; w.start = 'bar' }
-    no_stdout do
-      God.start
-    end
+    God.start
   end
   
   # at_exit
@@ -542,11 +514,11 @@ class TestGod < Test::Unit::TestCase
   # pattern_match
   
   def test_pattern_match
-    list = %w{ mongrel-3000 mongrel-3001 fuzed fuzed2 apache mysql}
+    list = %w{ mongrel-3000 mongrel-3001 fuzed22 fuzed fuzed2 apache mysql}
     
     assert_equal %w{ mongrel-3000 }, God.pattern_match('m3000', list)
     assert_equal %w{ mongrel-3001 }, God.pattern_match('m31', list)
-    assert_equal %w{ fuzed fuzed2 }, God.pattern_match('fu', list)
+    assert_equal %w{ fuzed fuzed2 fuzed22}, God.pattern_match('fu', list)
     assert_equal %w{ mysql }, God.pattern_match('sql', list)
   end
 end
diff -pruN 0.7.13-3/test/test_jabber.rb 0.7.18-1/test/test_jabber.rb
--- 0.7.13-3/test/test_jabber.rb	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/test/test_jabber.rb	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,36 @@
+#!/usr/bin/env ruby
+require File.dirname(__FILE__) + '/helper'
+
+class TestJabber < Test::Unit::TestCase
+
+  def setup
+    God::Contacts::Jabber.settings = { 
+      :jabber_id => 'test@example.com',
+      :password => 'pass'
+    }
+    @jabber = God::Contacts::Jabber.new
+    @jabber.jabber_id = 'recipient@example.com'
+  end
+
+  def test_notify
+    assert_nothing_raised do
+      God::Contacts::Jabber.any_instance.expects(:connect!).once.returns(nil)
+      God::Contacts::Jabber.any_instance.expects(:send!).once.returns(nil)
+      @jabber.notify(:a, :b, :c, :d, :e)
+      assert_equal "sent jabber message to recipient@example.com", @jabber.info
+    end
+  end
+  
+  # def test_live_notify
+  #   God::Contacts::Jabber.settings = { 
+  #     :jabber_id => 'real_user@example.com',
+  #     :password => 'pass'
+  #   }
+  #   recipient = "real_recipient@example.com"
+  #   
+  #   jabber = God::Contacts::Jabber.new
+  #   jabber.jabber_id = recipient
+  #   jabber.notify("Hello", Time.now, "Test", "Test", "localhost")
+  #   assert_equal "sent jabber message to #{recipient}", jabber.info
+  # end
+end
\ No newline at end of file
diff -pruN 0.7.13-3/test/test_logger.rb 0.7.18-1/test/test_logger.rb
--- 0.7.13-3/test/test_logger.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_logger.rb	2010-02-21 14:01:37.000000000 +0000
@@ -2,17 +2,16 @@ require File.dirname(__FILE__) + '/helpe
 
 class TestLogger < Test::Unit::TestCase
   def setup
-    @log = God::Logger.new
+    @log = God::Logger.new(StringIO.new('/dev/null'))
   end
   
   # log
   
-  def test_log
+  def test_log_should_keep_logs_when_wanted
+    @log.watch_log_since('foo', Time.now)
     @log.expects(:info).with("qux")
     
-    no_stdout do
-      @log.log(stub(:name => 'foo'), :info, "qux")
-    end
+    @log.log(stub(:name => 'foo'), :info, "qux")
     
     assert_equal 1, @log.logs.size
     assert_instance_of Time, @log.logs['foo'][0][0]
@@ -21,10 +20,7 @@ class TestLogger < Test::Unit::TestCase
   
   def test_log_should_send_to_syslog
     Syslog.expects(:crit).with('foo')
-    
-    no_stdout do
-      @log.log(stub(:name => 'foo'), :fatal, "foo")
-    end
+    @log.log(stub(:name => 'foo'), :fatal, "foo")
   end
   
   # watch_log_since
@@ -32,18 +28,16 @@ class TestLogger < Test::Unit::TestCase
   def test_watch_log_since
     t1 = Time.now
     
-    no_stdout do
-      @log.log(stub(:name => 'foo'), :info, "one")
-      @log.log(stub(:name => 'foo'), :info, "two")
-    end
+    @log.watch_log_since('foo', t1)
+    
+    @log.log(stub(:name => 'foo'), :info, "one")
+    @log.log(stub(:name => 'foo'), :info, "two")
     
     assert_match(/one.*two/m, @log.watch_log_since('foo', t1))
     
     t2 = Time.now
     
-    no_stdout do
-      @log.log(stub(:name => 'foo'), :info, "three")
-    end
+    @log.log(stub(:name => 'foo'), :info, "three")
     
     out = @log.watch_log_since('foo', t2)
     
@@ -55,9 +49,7 @@ class TestLogger < Test::Unit::TestCase
   # regular methods
   
   def test_fatal
-    no_stdout do
-      @log.fatal('foo')
-    end
+    @log.fatal('foo')
     assert_equal 0, @log.logs.size
   end
 end
\ No newline at end of file
diff -pruN 0.7.13-3/test/test_process.rb 0.7.18-1/test/test_process.rb
--- 0.7.13-3/test/test_process.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_process.rb	2010-02-21 14:01:37.000000000 +0000
@@ -53,11 +53,7 @@ class TestProcessChild < Test::Unit::Tes
     @p.log = '/tmp/foo.log'
     @p.uid = 'foobarbaz'
     
-    no_stdout do
-      no_stderr do
-        assert !@p.valid?
-      end
-    end
+    assert !@p.valid?
   end
   
   def test_valid_should_return_true_if_gid_exists
@@ -73,21 +69,37 @@ class TestProcessChild < Test::Unit::Tes
     @p.log = '/tmp/foo.log'
     @p.gid = 'foobarbaz'
     
-    no_stdout do
-      no_stderr do
-        assert !@p.valid?
-      end
-    end
+    assert !@p.valid?
+  end
+
+  def test_valid_should_return_true_if_dir_exists
+    @p.start = 'qux'
+    @p.log = '/tmp/foo.log'
+    @p.dir = '/tmp'
+
+    assert @p.valid?
+  end
+
+  def test_valid_should_return_false_if_dir_does_not_exists
+    @p.start = 'qux'
+    @p.log = '/tmp/foo.log'
+    @p.dir = '/tmp/doesnotexist'
+
+    assert !@p.valid?
+  end
+
+  def test_valid_should_return_false_if_dir_is_not_a_dir
+    @p.start = 'qux'
+    @p.log = '/tmp/foo.log'
+    @p.dir = '/etc/passwd'
+
+    assert !@p.valid?
   end
 
   def test_valid_should_return_false_with_bogus_chroot
     @p.chroot = '/bogusroot'
 
-    no_stdout do
-      no_stderr do
-        assert !@p.valid?
-      end
-    end
+    assert !@p.valid?
   end
 
   def test_valid_should_return_true_with_chroot_and_valid_log
@@ -152,18 +164,7 @@ class TestProcessDaemon < Test::Unit::Te
   def test_valid_should_return_false_if_no_start
     @p.name = 'foo'
     @p.stop = 'baz'
-    no_stdout do
-      assert !@p.valid?
-    end
-  end
-  
-  def test_valid_should_return_false_if_self_daemonized_and_no_stop
-    @p.start = 'bar'
-    @p.pid_file = 'foo'
-    
-    no_stdout do
-      assert !@p.valid?
-    end
+    assert !@p.valid?
   end
   
   # pid
diff -pruN 0.7.13-3/test/test_socket.rb 0.7.18-1/test/test_socket.rb
--- 0.7.13-3/test/test_socket.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_socket.rb	2010-02-21 14:01:37.000000000 +0000
@@ -9,23 +9,17 @@ class TestSocket < Test::Unit::TestCase
 
   def test_should_start_a_drb_server
     DRb.expects(:start_service)
-    no_stdout do
-      God::Socket.new
-    end
+    God::Socket.new
   end
 
   def test_should_use_supplied_port_and_host
     DRb.expects(:start_service).with { |uri, object| uri == "drbunix:///tmp/god.9999.sock" && object.is_a?(God::Socket) }
-    no_stdout do
-      server = God::Socket.new(9999)
-    end
+    server = God::Socket.new(9999)
   end
 
   def test_should_forward_foreign_method_calls_to_god
     server = nil
-    no_stdout do
-      server = God::Socket.new
-    end
+    server = God::Socket.new
     God.expects(:send).with(:something_random)
     server.something_random
   end
@@ -34,9 +28,7 @@ class TestSocket < Test::Unit::TestCase
   
   def test_ping_should_return_true
     server = nil
-    no_stdout do
-      server = God::Socket.new
-    end
+    server = God::Socket.new
     assert server.ping
   end
 end
diff -pruN 0.7.13-3/test/test_task.rb 0.7.18-1/test/test_task.rb
--- 0.7.13-3/test/test_task.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_task.rb	2010-02-21 14:01:37.000000000 +0000
@@ -15,23 +15,17 @@ class TestTask < Test::Unit::TestCase
   
   def test_valid_should_return_false_if_no_name
     @task.name = nil
-    no_stdout do
-      assert !@task.valid?
-    end
+    assert !@task.valid?
   end
   
   def test_valid_should_return_false_if_no_valid_states
     @task.valid_states = nil
-    no_stdout do
-      assert !@task.valid?
-    end
+    assert !@task.valid?
   end
   
   def test_valid_should_return_false_if_no_initial_state
     @task.initial_state = nil
-    no_stdout do
-      assert !@task.valid?
-    end
+    assert !@task.valid?
   end
   
   # transition
@@ -69,14 +63,14 @@ class TestTask < Test::Unit::TestCase
     @task.foo = 'foo'
     Thread.current.stubs(:==).returns(true)
     @task.expects(:system).with('foo')
-    no_stdout { @task.action(:foo, nil) }
+    @task.action(:foo, nil)
   end
   
   def test_action_should_call_lambda_commands
     @task.foo = lambda { }
     Thread.current.stubs(:==).returns(true)
     @task.foo.expects(:call)
-    no_stdout { @task.action(:foo, nil) }
+    @task.action(:foo, nil)
   end
   
   def test_action_should_raise_not_implemented_on_non_string_or_lambda_action
@@ -142,10 +136,7 @@ class TestTask < Test::Unit::TestCase
     
     c.expects(:test).returns(false)
     @task.driver.expects(:schedule)
-    
-    no_stdout do
-      @task.handle_poll(c)
-    end
+    @task.handle_poll(c)
   end
   
   def test_handle_poll_change_should_move
@@ -158,10 +149,7 @@ class TestTask < Test::Unit::TestCase
     
     c.expects(:test).returns(true)
     @task.expects(:move).with(:up)
-    
-    no_stdout do
-      @task.handle_poll(c)
-    end
+    @task.handle_poll(c)
   end
   
   def test_handle_poll_should_use_overridden_transition
@@ -175,10 +163,7 @@ class TestTask < Test::Unit::TestCase
     @task.directory[c] = m
     
     @task.expects(:move).with(:start)
-    
-    no_stdout do
-      @task.handle_poll(c)
-    end
+    @task.handle_poll(c)
   end
   
   def test_handle_poll_should_notify_if_triggering
@@ -192,10 +177,7 @@ class TestTask < Test::Unit::TestCase
     
     c.expects(:test).returns(true)
     @task.expects(:notify)
-    
-    no_stdout do
-      @task.handle_poll(c)
-    end
+    @task.handle_poll(c)
   end
   
   def test_handle_poll_should_not_notify_if_not_triggering
@@ -209,10 +191,21 @@ class TestTask < Test::Unit::TestCase
     
     c.expects(:test).returns(false)
     @task.expects(:notify).never
+    @task.handle_poll(c)
+  end
+  
+  def test_handle_poll_unexpected_exception_should_reschedule
+    c = Conditions::FakePollCondition.new
+    c.watch = @task
+    c.interval = 10
     
-    no_stdout do
-      @task.handle_poll(c)
-    end
+    m = Metric.new(@task, {true => :up})
+    @task.directory[c] = m
+    
+    c.expects(:test).raises(StandardError)
+    @task.driver.expects(:schedule)
+    
+    @task.handle_poll(c)
   end
   
   # handle_event
@@ -225,10 +218,7 @@ class TestTask < Test::Unit::TestCase
     @task.directory[c] = m
     
     @task.expects(:move).with(:up)
-    
-    no_stdout do
-      @task.handle_event(c)
-    end
+    @task.handle_event(c)
   end
   
   def test_handle_event_should_notify_if_triggering
@@ -240,10 +230,7 @@ class TestTask < Test::Unit::TestCase
     @task.directory[c] = m
     
     @task.expects(:notify)
-    
-    no_stdout do
-      @task.handle_event(c)
-    end
+    @task.handle_event(c)
   end
   
   def test_handle_event_should_not_notify_if_no_notify_set
@@ -254,9 +241,6 @@ class TestTask < Test::Unit::TestCase
     @task.directory[c] = m
     
     @task.expects(:notify).never
-    
-    no_stdout do
-      @task.handle_event(c)
-    end
+    @task.handle_event(c)
   end
 end
\ No newline at end of file
diff -pruN 0.7.13-3/test/test_watch.rb 0.7.18-1/test/test_watch.rb
--- 0.7.13-3/test/test_watch.rb	2008-11-13 20:31:30.000000000 +0000
+++ 0.7.18-1/test/test_watch.rb	2010-02-21 14:01:37.000000000 +0000
@@ -143,7 +143,7 @@ class TestWatch < Test::Unit::TestCase
     
     metric.expects(:disable).never
     
-    no_stdout { @watch.move(:init) }
+    @watch.move(:init)
   end
   
   def test_move_should_clean_up_from_state_if_not_nil
@@ -162,11 +162,11 @@ class TestWatch < Test::Unit::TestCase
       end
     end
     
-    no_stdout { @watch.move(:init) }
+    @watch.move(:init)
     
     metric.expects(:disable)
     
-    no_stdout { @watch.move(:up) }
+    @watch.move(:up)
   end
   
   def test_move_should_call_action
@@ -175,7 +175,7 @@ class TestWatch < Test::Unit::TestCase
     
     @watch.expects(:action).with(:start)
     
-    no_stdout { @watch.move(:start) }
+    @watch.move(:start)
   end
   
   def test_move_should_move_to_up_state_if_no_start_or_restart_metric
@@ -184,7 +184,7 @@ class TestWatch < Test::Unit::TestCase
     
     [:start, :restart].each do |state|
       @watch.expects(:action)
-      no_stdout { @watch.move(state) }
+      @watch.move(state)
       assert_equal :up, @watch.state
     end
   end
@@ -207,7 +207,7 @@ class TestWatch < Test::Unit::TestCase
     
     metric.expects(:enable)
     
-    no_stdout { @watch.move(:init) }
+    @watch.move(:init)
   end
   
   # action
@@ -248,7 +248,7 @@ class TestWatch < Test::Unit::TestCase
   def test_call_action
     c = Conditions::FakePollCondition.new
     God::Process.any_instance.expects(:call_action).with(:start)
-    no_stdout { @watch.call_action(c, :start) }
+    @watch.call_action(c, :start)
   end
   
   def test_call_action_should_call_before_start_when_behavior_has_that
@@ -256,7 +256,7 @@ class TestWatch < Test::Unit::TestCase
     c = Conditions::FakePollCondition.new
     God::Process.any_instance.expects(:call_action).with(:start)
     Behaviors::FakeBehavior.any_instance.expects(:before_start)
-    no_stdout { @watch.call_action(c, :start) }
+    @watch.call_action(c, :start)
   end
   
   def test_call_action_should_call_after_start_when_behavior_has_that
@@ -264,7 +264,7 @@ class TestWatch < Test::Unit::TestCase
     c = Conditions::FakePollCondition.new
     God::Process.any_instance.expects(:call_action).with(:start)
     Behaviors::FakeBehavior.any_instance.expects(:after_start)
-    no_stdout { @watch.call_action(c, :start) }
+    @watch.call_action(c, :start)
   end
   
   # canonical_hash_form
Binary files 0.7.13-3/test/._test_webhook.rb and 0.7.18-1/test/._test_webhook.rb differ
diff -pruN 0.7.13-3/VERSION.yml 0.7.18-1/VERSION.yml
--- 0.7.13-3/VERSION.yml	1970-01-01 01:00:00.000000000 +0100
+++ 0.7.18-1/VERSION.yml	2010-02-21 14:01:37.000000000 +0000
@@ -0,0 +1,4 @@
+--- 
+:major: 0
+:minor: 7
+:patch: 18
